# encoding: utf-8

"""
Step implementations for paragraph format-related features.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then, when

from docx import Document
from docx.enum.text import WD_ALIGN_PARAGRAPH, WD_LINE_SPACING
from docx.shared import Pt
from docx.text.tabstops import TabStops

from helpers import test_docx


# given ===================================================


@given("a paragraph format")
def given_a_paragraph_format(context):
    document = Document(test_docx("tab-stops"))
    context.paragraph_format = document.paragraphs[0].paragraph_format


@given("a paragraph format having {prop_name} set {setting}")
def given_a_paragraph_format_having_prop_set(context, prop_name, setting):
    style_name = {
        "to inherit": "Normal",
        "On": "Base",
        "Off": "Citation",
    }[setting]
    document = Document(test_docx("sty-known-styles"))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given("a paragraph format having {setting} line spacing")
def given_a_paragraph_format_having_setting_line_spacing(context, setting):
    style_name = {
        "inherited": "Normal",
        "14 pt": "Base",
        "double": "Citation",
    }[setting]
    document = Document(test_docx("sty-known-styles"))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given("a paragraph format having {setting} space {side}")
def given_a_paragraph_format_having_setting_spacing(context, setting, side):
    style_name = "Normal" if setting == "inherited" else "Base"
    document = Document(test_docx("sty-known-styles"))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given("a paragraph format having {type} alignment")
def given_a_paragraph_format_having_align_type_alignment(context, type):
    style_name = {
        "inherited": "Normal",
        "center": "Base",
        "right": "Citation",
    }[type]
    document = Document(test_docx("sty-known-styles"))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given("a paragraph format having {type} indent of {value}")
def given_a_paragraph_format_having_type_indent_value(context, type, value):
    style_name = {
        "inherit": "Normal",
        "18 pt": "Base",
        "17.3 pt": "Base",
        "-17.3 pt": "Citation",
        "46.1 pt": "Citation",
    }[value]
    document = Document(test_docx("sty-known-styles"))
    context.paragraph_format = document.styles[style_name].paragraph_format


# when ====================================================


@when("I assign {value} to paragraph_format.line_spacing")
def when_I_assign_value_to_paragraph_format_line_spacing(context, value):
    new_value = {
        "Pt(14)": Pt(14),
        "2": 2,
    }.get(value)
    new_value = float(value) if new_value is None else new_value
    context.paragraph_format.line_spacing = new_value


@when("I assign {value} to paragraph_format.line_spacing_rule")
def when_I_assign_value_to_paragraph_format_line_rule(context, value):
    new_value = {
        "None": None,
        "WD_LINE_SPACING.EXACTLY": WD_LINE_SPACING.EXACTLY,
        "WD_LINE_SPACING.MULTIPLE": WD_LINE_SPACING.MULTIPLE,
        "WD_LINE_SPACING.SINGLE": WD_LINE_SPACING.SINGLE,
        "WD_LINE_SPACING.DOUBLE": WD_LINE_SPACING.DOUBLE,
        "WD_LINE_SPACING.AT_LEAST": WD_LINE_SPACING.AT_LEAST,
        "WD_LINE_SPACING.ONE_POINT_FIVE": WD_LINE_SPACING.ONE_POINT_FIVE,
    }[value]
    paragraph_format = context.paragraph_format
    paragraph_format.line_spacing_rule = new_value


@when("I assign {value} to paragraph_format.alignment")
def when_I_assign_value_to_paragraph_format_alignment(context, value):
    new_value = {
        "None": None,
        "WD_ALIGN_PARAGRAPH.CENTER": WD_ALIGN_PARAGRAPH.CENTER,
        "WD_ALIGN_PARAGRAPH.RIGHT": WD_ALIGN_PARAGRAPH.RIGHT,
    }[value]
    paragraph_format = context.paragraph_format
    paragraph_format.alignment = new_value


@when("I assign {value} to paragraph_format.space_{side}")
def when_I_assign_value_to_paragraph_format_space(context, value, side):
    paragraph_format = context.paragraph_format
    prop_name = "space_%s" % side
    new_value = {
        "None": None,
        "Pt(12)": Pt(12),
        "Pt(18)": Pt(18),
    }[value]
    setattr(paragraph_format, prop_name, new_value)


@when("I assign {value} to paragraph_format.{type_}_indent")
def when_I_assign_value_to_paragraph_format_indent(context, value, type_):
    paragraph_format = context.paragraph_format
    prop_name = "%s_indent" % type_
    value = None if value == "None" else Pt(float(value.split()[0]))
    setattr(paragraph_format, prop_name, value)


@when("I assign {value} to paragraph_format.{prop_name}")
def when_I_assign_value_to_paragraph_format_prop(context, value, prop_name):
    paragraph_format = context.paragraph_format
    value = {"None": None, "True": True, "False": False}[value]
    setattr(paragraph_format, prop_name, value)


# then =====================================================


@then("paragraph_format.tab_stops is a TabStops object")
def then_paragraph_format_tab_stops_is_a_tabstops_object(context):
    tab_stops = context.paragraph_format.tab_stops
    assert isinstance(tab_stops, TabStops)


@then("paragraph_format.alignment is {value}")
def then_paragraph_format_alignment_is_value(context, value):
    expected_value = {
        "None": None,
        "WD_ALIGN_PARAGRAPH.LEFT": WD_ALIGN_PARAGRAPH.LEFT,
        "WD_ALIGN_PARAGRAPH.CENTER": WD_ALIGN_PARAGRAPH.CENTER,
        "WD_ALIGN_PARAGRAPH.RIGHT": WD_ALIGN_PARAGRAPH.RIGHT,
    }[value]
    paragraph_format = context.paragraph_format
    assert paragraph_format.alignment == expected_value


@then("paragraph_format.line_spacing is {value}")
def then_paragraph_format_line_spacing_is_value(context, value):
    expected_value = (
        None if value == "None" else float(value) if "." in value else int(value)
    )
    paragraph_format = context.paragraph_format

    if expected_value is None or isinstance(expected_value, int):
        assert paragraph_format.line_spacing == expected_value
    else:
        assert abs(paragraph_format.line_spacing - expected_value) < 0.001


@then("paragraph_format.line_spacing_rule is {value}")
def then_paragraph_format_line_spacing_rule_is_value(context, value):
    expected_value = {
        "None": None,
        "WD_LINE_SPACING.EXACTLY": WD_LINE_SPACING.EXACTLY,
        "WD_LINE_SPACING.MULTIPLE": WD_LINE_SPACING.MULTIPLE,
        "WD_LINE_SPACING.SINGLE": WD_LINE_SPACING.SINGLE,
        "WD_LINE_SPACING.DOUBLE": WD_LINE_SPACING.DOUBLE,
        "WD_LINE_SPACING.AT_LEAST": WD_LINE_SPACING.AT_LEAST,
        "WD_LINE_SPACING.ONE_POINT_FIVE": WD_LINE_SPACING.ONE_POINT_FIVE,
    }[value]
    paragraph_format = context.paragraph_format
    assert paragraph_format.line_spacing_rule == expected_value


@then("paragraph_format.space_{side} is {value}")
def then_paragraph_format_space_side_is_value(context, side, value):
    expected_value = None if value == "None" else int(value)
    prop_name = "space_%s" % side
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value


@then("paragraph_format.{type_}_indent is {value}")
def then_paragraph_format_type_indent_is_value(context, type_, value):
    expected_value = None if value == "None" else int(value)
    prop_name = "%s_indent" % type_
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value


@then("paragraph_format.{prop_name} is {value}")
def then_paragraph_format_prop_name_is_value(context, prop_name, value):
    expected_value = {"None": None, "True": True, "False": False}[value]
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value



# encoding: utf-8

"""Styles object, container for all objects in the styles part"""

from __future__ import absolute_import, division, print_function, unicode_literals

from warnings import warn

from docx.shared import ElementProxy
from docx.styles import BabelFish
from docx.styles.latent import LatentStyles
from docx.styles.style import BaseStyle, StyleFactory


class Styles(ElementProxy):
    """Provides access to the styles defined in a document.

    Accessed using the :attr:`.Document.styles` property. Supports ``len()``, iteration,
    and dictionary-style access by style name.
    """

    __slots__ = ()

    def __contains__(self, name):
        """
        Enables `in` operator on style name.
        """
        internal_name = BabelFish.ui2internal(name)
        for style in self._element.style_lst:
            if style.name_val == internal_name:
                return True
        return False

    def __getitem__(self, key):
        """
        Enables dictionary-style access by UI name. Lookup by style id is
        deprecated, triggers a warning, and will be removed in a near-future
        release.
        """
        style_elm = self._element.get_by_name(BabelFish.ui2internal(key))
        if style_elm is not None:
            return StyleFactory(style_elm)

        style_elm = self._element.get_by_id(key)
        if style_elm is not None:
            msg = (
                "style lookup by style_id is deprecated. Use style name as "
                "key instead."
            )
            warn(msg, UserWarning, stacklevel=2)
            return StyleFactory(style_elm)

        raise KeyError("no style with name '%s'" % key)

    def __iter__(self):
        return (StyleFactory(style) for style in self._element.style_lst)

    def __len__(self):
        return len(self._element.style_lst)

    def add_style(self, name, style_type, builtin=False):
        """
        Return a newly added style object of *style_type* and identified
        by *name*. A builtin style can be defined by passing True for the
        optional *builtin* argument.
        """
        style_name = BabelFish.ui2internal(name)
        if style_name in self:
            raise ValueError("document already contains style '%s'" % name)
        style = self._element.add_style_of_type(style_name, style_type, builtin)
        return StyleFactory(style)

    def default(self, style_type):
        """
        Return the default style for *style_type* or |None| if no default is
        defined for that type (not common).
        """
        style = self._element.default_for(style_type)
        if style is None:
            return None
        return StyleFactory(style)

    def get_by_id(self, style_id, style_type):
        """Return the style of *style_type* matching *style_id*.

        Returns the default for *style_type* if *style_id* is not found or is |None|, or
        if the style having *style_id* is not of *style_type*.
        """
        if style_id is None:
            return self.default(style_type)
        return self._get_by_id(style_id, style_type)

    def get_style_id(self, style_or_name, style_type):
        """
        Return the id of the style corresponding to *style_or_name*, or
        |None| if *style_or_name* is |None|. If *style_or_name* is not
        a style object, the style is looked up using *style_or_name* as
        a style name, raising |ValueError| if no style with that name is
        defined. Raises |ValueError| if the target style is not of
        *style_type*.
        """
        if style_or_name is None:
            return None
        elif isinstance(style_or_name, BaseStyle):
            return self._get_style_id_from_style(style_or_name, style_type)
        else:
            return self._get_style_id_from_name(style_or_name, style_type)

    @property
    def latent_styles(self):
        """
        A |LatentStyles| object providing access to the default behaviors for
        latent styles and the collection of |_LatentStyle| objects that
        define overrides of those defaults for a particular named latent
        style.
        """
        return LatentStyles(self._element.get_or_add_latentStyles())

    def _get_by_id(self, style_id, style_type):
        """
        Return the style of *style_type* matching *style_id*. Returns the
        default for *style_type* if *style_id* is not found or if the style
        having *style_id* is not of *style_type*.
        """
        style = self._element.get_by_id(style_id)
        if style is None or style.type != style_type:
            return self.default(style_type)
        return StyleFactory(style)

    def _get_style_id_from_name(self, style_name, style_type):
        """
        Return the id of the style of *style_type* corresponding to
        *style_name*. Returns |None| if that style is the default style for
        *style_type*. Raises |ValueError| if the named style is not found in
        the document or does not match *style_type*.
        """
        return self._get_style_id_from_style(self[style_name], style_type)

    def _get_style_id_from_style(self, style, style_type):
        """
        Return the id of *style*, or |None| if it is the default style of
        *style_type*. Raises |ValueError| if style is not of *style_type*.
        """
        if style.type != style_type:
            raise ValueError(
                "assigned style is type %s, need type %s" % (style.type, style_type)
            )
        if style == self.default(style_type):
            return None
        return style.style_id



# encoding: utf-8

"""
Enumerations used by DrawingML objects
"""

from __future__ import absolute_import

from .base import alias, Enumeration, EnumMember, XmlEnumeration, XmlMappedEnumMember


class MSO_COLOR_TYPE(Enumeration):
    """
    Specifies the color specification scheme

    Example::

        from docx.enum.dml import MSO_COLOR_TYPE

        assert font.color.type == MSO_COLOR_TYPE.SCHEME
    """

    __ms_name__ = "MsoColorType"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff864912(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember("RGB", 1, "Color is specified by an |RGBColor| value."),
        EnumMember("THEME", 2, "Color is one of the preset theme colors."),
        EnumMember(
            "AUTO", 101, "Color is determined automatically by the " "application."
        ),
    )


@alias("MSO_THEME_COLOR")
class MSO_THEME_COLOR_INDEX(XmlEnumeration):
    """
    Indicates the Office theme color, one of those shown in the color gallery
    on the formatting ribbon.

    Alias: ``MSO_THEME_COLOR``

    Example::

        from docx.enum.dml import MSO_THEME_COLOR

        font.color.theme_color = MSO_THEME_COLOR.ACCENT_1
    """

    __ms_name__ = "MsoThemeColorIndex"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff860782(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember("NOT_THEME_COLOR", 0, "Indicates the color is not a theme color."),
        XmlMappedEnumMember(
            "ACCENT_1", 5, "accent1", "Specifies the Accent 1 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_2", 6, "accent2", "Specifies the Accent 2 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_3", 7, "accent3", "Specifies the Accent 3 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_4", 8, "accent4", "Specifies the Accent 4 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_5", 9, "accent5", "Specifies the Accent 5 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_6", 10, "accent6", "Specifies the Accent 6 theme color."
        ),
        XmlMappedEnumMember(
            "BACKGROUND_1",
            14,
            "background1",
            "Specifies the Background 1 " "theme color.",
        ),
        XmlMappedEnumMember(
            "BACKGROUND_2",
            16,
            "background2",
            "Specifies the Background 2 " "theme color.",
        ),
        XmlMappedEnumMember("DARK_1", 1, "dark1", "Specifies the Dark 1 theme color."),
        XmlMappedEnumMember("DARK_2", 3, "dark2", "Specifies the Dark 2 theme color."),
        XmlMappedEnumMember(
            "FOLLOWED_HYPERLINK",
            12,
            "followedHyperlink",
            "Specifies the " "theme color for a clicked hyperlink.",
        ),
        XmlMappedEnumMember(
            "HYPERLINK",
            11,
            "hyperlink",
            "Specifies the theme color for a " "hyperlink.",
        ),
        XmlMappedEnumMember(
            "LIGHT_1", 2, "light1", "Specifies the Light 1 theme color."
        ),
        XmlMappedEnumMember(
            "LIGHT_2", 4, "light2", "Specifies the Light 2 theme color."
        ),
        XmlMappedEnumMember("TEXT_1", 13, "text1", "Specifies the Text 1 theme color."),
        XmlMappedEnumMember("TEXT_2", 15, "text2", "Specifies the Text 2 theme color."),
    )



import numbers

from lxml import etree

NS_VT = "http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes"


class CustomProperties:
    """
    Corresponds to part named ``/docProps/custom.xml``, containing the custom
    document properties for this document package.
    """

    def __init__(self, element):
        self._element = element

    def __getitem__(self, item):
        prop = self.lookup(item)
        if prop is not None:
            elm = prop[0]
            if elm.tag == "{%s}i4" % NS_VT:
                try:
                    return int(elm.text)
                except Exception:
                    return elm.text
            elif elm.tag == "{%s}bool" % NS_VT:
                return elm.text == "1"
            return elm.text

    def __setitem__(self, key, value):
        prop = self.lookup(key)
        if prop is None:
            elmType = "lpwstr"
            if isinstance(value, bool):
                elmType = "bool"
                value = str(1 if value else 0)
            elif isinstance(value, numbers.Number):
                elmType = "i4"
                value = str(int(value))
            prop = etree.SubElement(self._element, "property")
            elm = etree.SubElement(
                prop, "{%s}%s" % (NS_VT, elmType), nsmap={"vt": NS_VT}
            )
            elm.text = value
            prop.set("name", key)
            prop.set("fmtid", "{D5CDD505-2E9C-101B-9397-08002B2CF9AE}")
            prop.set("pid", f"{str(len(self._element) + 1)}")
        else:
            elm = prop[0]
            if elm.tag == "{%s}i4" % NS_VT:
                elm.text = str(int(value))
            elif elm.tag == "{%s}bool" % NS_VT:
                elm.text = str(1 if value else 0)
            else:
                elm.text = f"{str(value)}"

    def __len__(self):
        return len(self._element)

    def lookup(self, item):
        return next(
            (child for child in self._element if child.get("name") == item), None
        )



# encoding: utf-8

"""
Test data builders for styles part XML elements
"""

from ...unitdata import BaseBuilder


class CT_StyleBuilder(BaseBuilder):
    __tag__ = "w:style"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:type", "w:styleId", "w:default", "w:customStyle")


class CT_StylesBuilder(BaseBuilder):
    __tag__ = "w:styles"
    __nspfxs__ = ("w",)
    __attrs__ = ()


def a_style():
    return CT_StyleBuilder()


def a_styles():
    return CT_StylesBuilder()



# encoding: utf-8

"""
Enumerations related to styles
"""

from __future__ import absolute_import, print_function, unicode_literals

from .base import alias, EnumMember, XmlEnumeration, XmlMappedEnumMember


@alias("WD_STYLE")
class WD_BUILTIN_STYLE(XmlEnumeration):
    """
    alias: **WD_STYLE**

    Specifies a built-in Microsoft Word style.

    Example::

        from docx import Document
        from docx.enum.style import WD_STYLE

        document = Document()
        styles = document.styles
        style = styles[WD_STYLE.BODY_TEXT]
    """

    __ms_name__ = "WdBuiltinStyle"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff835210.aspx"

    __members__ = (
        EnumMember("BLOCK_QUOTATION", -85, "Block Text."),
        EnumMember("BODY_TEXT", -67, "Body Text."),
        EnumMember("BODY_TEXT_2", -81, "Body Text 2."),
        EnumMember("BODY_TEXT_3", -82, "Body Text 3."),
        EnumMember("BODY_TEXT_FIRST_INDENT", -78, "Body Text First Indent."),
        EnumMember("BODY_TEXT_FIRST_INDENT_2", -79, "Body Text First Indent 2."),
        EnumMember("BODY_TEXT_INDENT", -68, "Body Text Indent."),
        EnumMember("BODY_TEXT_INDENT_2", -83, "Body Text Indent 2."),
        EnumMember("BODY_TEXT_INDENT_3", -84, "Body Text Indent 3."),
        EnumMember("BOOK_TITLE", -265, "Book Title."),
        EnumMember("CAPTION", -35, "Caption."),
        EnumMember("CLOSING", -64, "Closing."),
        EnumMember("COMMENT_REFERENCE", -40, "Comment Reference."),
        EnumMember("COMMENT_TEXT", -31, "Comment Text."),
        EnumMember("DATE", -77, "Date."),
        EnumMember("DEFAULT_PARAGRAPH_FONT", -66, "Default Paragraph Font."),
        EnumMember("EMPHASIS", -89, "Emphasis."),
        EnumMember("ENDNOTE_REFERENCE", -43, "Endnote Reference."),
        EnumMember("ENDNOTE_TEXT", -44, "Endnote Text."),
        EnumMember("ENVELOPE_ADDRESS", -37, "Envelope Address."),
        EnumMember("ENVELOPE_RETURN", -38, "Envelope Return."),
        EnumMember("FOOTER", -33, "Footer."),
        EnumMember("FOOTNOTE_REFERENCE", -39, "Footnote Reference."),
        EnumMember("FOOTNOTE_TEXT", -30, "Footnote Text."),
        EnumMember("HEADER", -32, "Header."),
        EnumMember("HEADING_1", -2, "Heading 1."),
        EnumMember("HEADING_2", -3, "Heading 2."),
        EnumMember("HEADING_3", -4, "Heading 3."),
        EnumMember("HEADING_4", -5, "Heading 4."),
        EnumMember("HEADING_5", -6, "Heading 5."),
        EnumMember("HEADING_6", -7, "Heading 6."),
        EnumMember("HEADING_7", -8, "Heading 7."),
        EnumMember("HEADING_8", -9, "Heading 8."),
        EnumMember("HEADING_9", -10, "Heading 9."),
        EnumMember("HTML_ACRONYM", -96, "HTML Acronym."),
        EnumMember("HTML_ADDRESS", -97, "HTML Address."),
        EnumMember("HTML_CITE", -98, "HTML Cite."),
        EnumMember("HTML_CODE", -99, "HTML Code."),
        EnumMember("HTML_DFN", -100, "HTML Definition."),
        EnumMember("HTML_KBD", -101, "HTML Keyboard."),
        EnumMember("HTML_NORMAL", -95, "Normal (Web)."),
        EnumMember("HTML_PRE", -102, "HTML Preformatted."),
        EnumMember("HTML_SAMP", -103, "HTML Sample."),
        EnumMember("HTML_TT", -104, "HTML Typewriter."),
        EnumMember("HTML_VAR", -105, "HTML Variable."),
        EnumMember("HYPERLINK", -86, "Hyperlink."),
        EnumMember("HYPERLINK_FOLLOWED", -87, "Followed Hyperlink."),
        EnumMember("INDEX_1", -11, "Index 1."),
        EnumMember("INDEX_2", -12, "Index 2."),
        EnumMember("INDEX_3", -13, "Index 3."),
        EnumMember("INDEX_4", -14, "Index 4."),
        EnumMember("INDEX_5", -15, "Index 5."),
        EnumMember("INDEX_6", -16, "Index 6."),
        EnumMember("INDEX_7", -17, "Index 7."),
        EnumMember("INDEX_8", -18, "Index 8."),
        EnumMember("INDEX_9", -19, "Index 9."),
        EnumMember("INDEX_HEADING", -34, "Index Heading"),
        EnumMember("INTENSE_EMPHASIS", -262, "Intense Emphasis."),
        EnumMember("INTENSE_QUOTE", -182, "Intense Quote."),
        EnumMember("INTENSE_REFERENCE", -264, "Intense Reference."),
        EnumMember("LINE_NUMBER", -41, "Line Number."),
        EnumMember("LIST", -48, "List."),
        EnumMember("LIST_2", -51, "List 2."),
        EnumMember("LIST_3", -52, "List 3."),
        EnumMember("LIST_4", -53, "List 4."),
        EnumMember("LIST_5", -54, "List 5."),
        EnumMember("LIST_BULLET", -49, "List Bullet."),
        EnumMember("LIST_BULLET_2", -55, "List Bullet 2."),
        EnumMember("LIST_BULLET_3", -56, "List Bullet 3."),
        EnumMember("LIST_BULLET_4", -57, "List Bullet 4."),
        EnumMember("LIST_BULLET_5", -58, "List Bullet 5."),
        EnumMember("LIST_CONTINUE", -69, "List Continue."),
        EnumMember("LIST_CONTINUE_2", -70, "List Continue 2."),
        EnumMember("LIST_CONTINUE_3", -71, "List Continue 3."),
        EnumMember("LIST_CONTINUE_4", -72, "List Continue 4."),
        EnumMember("LIST_CONTINUE_5", -73, "List Continue 5."),
        EnumMember("LIST_NUMBER", -50, "List Number."),
        EnumMember("LIST_NUMBER_2", -59, "List Number 2."),
        EnumMember("LIST_NUMBER_3", -60, "List Number 3."),
        EnumMember("LIST_NUMBER_4", -61, "List Number 4."),
        EnumMember("LIST_NUMBER_5", -62, "List Number 5."),
        EnumMember("LIST_PARAGRAPH", -180, "List Paragraph."),
        EnumMember("MACRO_TEXT", -46, "Macro Text."),
        EnumMember("MESSAGE_HEADER", -74, "Message Header."),
        EnumMember("NAV_PANE", -90, "Document Map."),
        EnumMember("NORMAL", -1, "Normal."),
        EnumMember("NORMAL_INDENT", -29, "Normal Indent."),
        EnumMember("NORMAL_OBJECT", -158, "Normal (applied to an object)."),
        EnumMember("NORMAL_TABLE", -106, "Normal (applied within a table)."),
        EnumMember("NOTE_HEADING", -80, "Note Heading."),
        EnumMember("PAGE_NUMBER", -42, "Page Number."),
        EnumMember("PLAIN_TEXT", -91, "Plain Text."),
        EnumMember("QUOTE", -181, "Quote."),
        EnumMember("SALUTATION", -76, "Salutation."),
        EnumMember("SIGNATURE", -65, "Signature."),
        EnumMember("STRONG", -88, "Strong."),
        EnumMember("SUBTITLE", -75, "Subtitle."),
        EnumMember("SUBTLE_EMPHASIS", -261, "Subtle Emphasis."),
        EnumMember("SUBTLE_REFERENCE", -263, "Subtle Reference."),
        EnumMember("TABLE_COLORFUL_GRID", -172, "Colorful Grid."),
        EnumMember("TABLE_COLORFUL_LIST", -171, "Colorful List."),
        EnumMember("TABLE_COLORFUL_SHADING", -170, "Colorful Shading."),
        EnumMember("TABLE_DARK_LIST", -169, "Dark List."),
        EnumMember("TABLE_LIGHT_GRID", -161, "Light Grid."),
        EnumMember("TABLE_LIGHT_GRID_ACCENT_1", -175, "Light Grid Accent 1."),
        EnumMember("TABLE_LIGHT_LIST", -160, "Light List."),
        EnumMember("TABLE_LIGHT_LIST_ACCENT_1", -174, "Light List Accent 1."),
        EnumMember("TABLE_LIGHT_SHADING", -159, "Light Shading."),
        EnumMember("TABLE_LIGHT_SHADING_ACCENT_1", -173, "Light Shading Accent 1."),
        EnumMember("TABLE_MEDIUM_GRID_1", -166, "Medium Grid 1."),
        EnumMember("TABLE_MEDIUM_GRID_2", -167, "Medium Grid 2."),
        EnumMember("TABLE_MEDIUM_GRID_3", -168, "Medium Grid 3."),
        EnumMember("TABLE_MEDIUM_LIST_1", -164, "Medium List 1."),
        EnumMember("TABLE_MEDIUM_LIST_1_ACCENT_1", -178, "Medium List 1 Accent 1."),
        EnumMember("TABLE_MEDIUM_LIST_2", -165, "Medium List 2."),
        EnumMember("TABLE_MEDIUM_SHADING_1", -162, "Medium Shading 1."),
        EnumMember(
            "TABLE_MEDIUM_SHADING_1_ACCENT_1", -176, "Medium Shading 1 Accent 1."
        ),
        EnumMember("TABLE_MEDIUM_SHADING_2", -163, "Medium Shading 2."),
        EnumMember(
            "TABLE_MEDIUM_SHADING_2_ACCENT_1", -177, "Medium Shading 2 Accent 1."
        ),
        EnumMember("TABLE_OF_AUTHORITIES", -45, "Table of Authorities."),
        EnumMember("TABLE_OF_FIGURES", -36, "Table of Figures."),
        EnumMember("TITLE", -63, "Title."),
        EnumMember("TOAHEADING", -47, "TOA Heading."),
        EnumMember("TOC_1", -20, "TOC 1."),
        EnumMember("TOC_2", -21, "TOC 2."),
        EnumMember("TOC_3", -22, "TOC 3."),
        EnumMember("TOC_4", -23, "TOC 4."),
        EnumMember("TOC_5", -24, "TOC 5."),
        EnumMember("TOC_6", -25, "TOC 6."),
        EnumMember("TOC_7", -26, "TOC 7."),
        EnumMember("TOC_8", -27, "TOC 8."),
        EnumMember("TOC_9", -28, "TOC 9."),
    )


class WD_STYLE_TYPE(XmlEnumeration):
    """
    Specifies one of the four style types: paragraph, character, list, or
    table.

    Example::

        from docx import Document
        from docx.enum.style import WD_STYLE_TYPE

        styles = Document().styles
        assert styles[0].type == WD_STYLE_TYPE.PARAGRAPH
    """

    __ms_name__ = "WdStyleType"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff196870.aspx"

    __members__ = (
        XmlMappedEnumMember("CHARACTER", 2, "character", "Character style."),
        XmlMappedEnumMember("LIST", 4, "numbering", "List style."),
        XmlMappedEnumMember("PARAGRAPH", 1, "paragraph", "Paragraph style."),
        XmlMappedEnumMember("TABLE", 3, "table", "Table style."),
    )



# encoding: utf-8

"""
Temporary stand-in for main oxml module that came across with the
PackageReader transplant. Probably much will get replaced with objects from
the pptx.oxml.core and then this module will either get deleted or only hold
the package related custom element classes.
"""

from __future__ import absolute_import, print_function, unicode_literals

from lxml import etree

from .constants import NAMESPACE as NS, RELATIONSHIP_TARGET_MODE as RTM


# configure XML parser
element_class_lookup = etree.ElementNamespaceClassLookup()
oxml_parser = etree.XMLParser(remove_blank_text=True, resolve_entities=False)
oxml_parser.set_element_class_lookup(element_class_lookup)

nsmap = {
    "ct": NS.OPC_CONTENT_TYPES,
    "pr": NS.OPC_RELATIONSHIPS,
    "r": NS.OFC_RELATIONSHIPS,
}


# ===========================================================================
# functions
# ===========================================================================


def parse_xml(text):
    """
    ``etree.fromstring()`` replacement that uses oxml parser
    """
    return etree.fromstring(text, oxml_parser)


def qn(tag):
    """
    Stands for "qualified name", a utility function to turn a namespace
    prefixed tag name into a Clark-notation qualified tag name for lxml. For
    example, ``qn('p:cSld')`` returns ``'{http://schemas.../main}cSld'``.
    """
    prefix, tagroot = tag.split(":")
    uri = nsmap[prefix]
    return "{%s}%s" % (uri, tagroot)


def serialize_part_xml(part_elm):
    """
    Serialize *part_elm* etree element to XML suitable for storage as an XML
    part. That is to say, no insignificant whitespace added for readability,
    and an appropriate XML declaration added with UTF-8 encoding specified.
    """
    return etree.tostring(part_elm, encoding="UTF-8", standalone=True)


def serialize_for_reading(element):
    """
    Serialize *element* to human-readable XML suitable for tests. No XML
    declaration.
    """
    return etree.tostring(element, encoding="unicode", pretty_print=True)


# ===========================================================================
# Custom element classes
# ===========================================================================


class BaseOxmlElement(etree.ElementBase):
    """
    Base class for all custom element classes, to add standardized behavior
    to all classes in one place.
    """

    @property
    def xml(self):
        """
        Return XML string for this element, suitable for testing purposes.
        Pretty printed for readability and without an XML declaration at the
        top.
        """
        return serialize_for_reading(self)


class CT_Default(BaseOxmlElement):
    """
    ``<Default>`` element, specifying the default content type to be applied
    to a part with the specified extension.
    """

    @property
    def content_type(self):
        """
        String held in the ``ContentType`` attribute of this ``<Default>``
        element.
        """
        return self.get("ContentType")

    @property
    def extension(self):
        """
        String held in the ``Extension`` attribute of this ``<Default>``
        element.
        """
        return self.get("Extension")

    @staticmethod
    def new(ext, content_type):
        """
        Return a new ``<Default>`` element with attributes set to parameter
        values.
        """
        xml = '<Default xmlns="%s"/>' % nsmap["ct"]
        default = parse_xml(xml)
        default.set("Extension", ext)
        default.set("ContentType", content_type)
        return default


class CT_Override(BaseOxmlElement):
    """
    ``<Override>`` element, specifying the content type to be applied for a
    part with the specified partname.
    """

    @property
    def content_type(self):
        """
        String held in the ``ContentType`` attribute of this ``<Override>``
        element.
        """
        return self.get("ContentType")

    @staticmethod
    def new(partname, content_type):
        """
        Return a new ``<Override>`` element with attributes set to parameter
        values.
        """
        xml = '<Override xmlns="%s"/>' % nsmap["ct"]
        override = parse_xml(xml)
        override.set("PartName", partname)
        override.set("ContentType", content_type)
        return override

    @property
    def partname(self):
        """
        String held in the ``PartName`` attribute of this ``<Override>``
        element.
        """
        return self.get("PartName")


class CT_Relationship(BaseOxmlElement):
    """
    ``<Relationship>`` element, representing a single relationship from a
    source to a target part.
    """

    @staticmethod
    def new(rId, reltype, target, target_mode=RTM.INTERNAL):
        """
        Return a new ``<Relationship>`` element.
        """
        xml = '<Relationship xmlns="%s"/>' % nsmap["pr"]
        relationship = parse_xml(xml)
        relationship.set("Id", rId)
        relationship.set("Type", reltype)
        relationship.set("Target", target)
        if target_mode == RTM.EXTERNAL:
            relationship.set("TargetMode", RTM.EXTERNAL)
        return relationship

    @property
    def rId(self):
        """
        String held in the ``Id`` attribute of this ``<Relationship>``
        element.
        """
        return self.get("Id")

    @property
    def reltype(self):
        """
        String held in the ``Type`` attribute of this ``<Relationship>``
        element.
        """
        return self.get("Type")

    @property
    def target_ref(self):
        """
        String held in the ``Target`` attribute of this ``<Relationship>``
        element.
        """
        return self.get("Target")

    @property
    def target_mode(self):
        """
        String held in the ``TargetMode`` attribute of this
        ``<Relationship>`` element, either ``Internal`` or ``External``.
        Defaults to ``Internal``.
        """
        return self.get("TargetMode", RTM.INTERNAL)


class CT_Relationships(BaseOxmlElement):
    """
    ``<Relationships>`` element, the root element in a .rels file.
    """

    def add_rel(self, rId, reltype, target, is_external=False):
        """
        Add a child ``<Relationship>`` element with attributes set according
        to parameter values.
        """
        target_mode = RTM.EXTERNAL if is_external else RTM.INTERNAL
        relationship = CT_Relationship.new(rId, reltype, target, target_mode)
        self.append(relationship)

    @staticmethod
    def new():
        """
        Return a new ``<Relationships>`` element.
        """
        xml = '<Relationships xmlns="%s"/>' % nsmap["pr"]
        relationships = parse_xml(xml)
        return relationships

    @property
    def Relationship_lst(self):
        """
        Return a list containing all the ``<Relationship>`` child elements.
        """
        return self.findall(qn("pr:Relationship"))

    @property
    def xml(self):
        """
        Return XML string for this element, suitable for saving in a .rels
        stream, not pretty printed and with an XML declaration at the top.
        """
        return serialize_part_xml(self)


class CT_Types(BaseOxmlElement):
    """
    ``<Types>`` element, the container element for Default and Override
    elements in [Content_Types].xml.
    """

    def add_default(self, ext, content_type):
        """
        Add a child ``<Default>`` element with attributes set to parameter
        values.
        """
        default = CT_Default.new(ext, content_type)
        self.append(default)

    def add_override(self, partname, content_type):
        """
        Add a child ``<Override>`` element with attributes set to parameter
        values.
        """
        override = CT_Override.new(partname, content_type)
        self.append(override)

    @property
    def defaults(self):
        return self.findall(qn("ct:Default"))

    @staticmethod
    def new():
        """
        Return a new ``<Types>`` element.
        """
        xml = '<Types xmlns="%s"/>' % nsmap["ct"]
        types = parse_xml(xml)
        return types

    @property
    def overrides(self):
        return self.findall(qn("ct:Override"))


ct_namespace = element_class_lookup.get_namespace(nsmap["ct"])
ct_namespace["Default"] = CT_Default
ct_namespace["Override"] = CT_Override
ct_namespace["Types"] = CT_Types

pr_namespace = element_class_lookup.get_namespace(nsmap["pr"])
pr_namespace["Relationship"] = CT_Relationship
pr_namespace["Relationships"] = CT_Relationships



# encoding: utf-8

"""Unit test suite for the docx.section module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from docx.enum.section import WD_HEADER_FOOTER, WD_ORIENT, WD_SECTION
from docx.parts.document import DocumentPart
from docx.parts.hdrftr import FooterPart, HeaderPart
from docx.section import _BaseHeaderFooter, _Footer, _Header, Section, Sections
from docx.shared import Inches

from .unitutil.cxml import element, xml
from .unitutil.mock import call, class_mock, instance_mock, method_mock, property_mock


class DescribeSections(object):
    def it_knows_how_many_sections_it_contains(self):
        sections = Sections(
            element("w:document/w:body/(w:p/w:pPr/w:sectPr, w:sectPr)"), None
        )
        assert len(sections) == 2

    def it_can_iterate_over_its_Section_instances(
        self, Section_, section_, document_part_
    ):
        document_elm = element("w:document/w:body/(w:p/w:pPr/w:sectPr, w:sectPr)")
        sectPrs = document_elm.xpath("//w:sectPr")
        Section_.return_value = section_
        sections = Sections(document_elm, document_part_)

        section_lst = [s for s in sections]

        assert Section_.call_args_list == [
            call(sectPrs[0], document_part_),
            call(sectPrs[1], document_part_),
        ]
        assert section_lst == [section_, section_]

    def it_can_access_its_Section_instances_by_index(
        self, Section_, section_, document_part_
    ):
        document_elm = element(
            "w:document/w:body/(w:p/w:pPr/w:sectPr,w:p/w:pPr/w:sectPr,w:sectPr)"
        )
        sectPrs = document_elm.xpath("//w:sectPr")
        Section_.return_value = section_
        sections = Sections(document_elm, document_part_)

        section_lst = [sections[idx] for idx in range(3)]

        assert Section_.call_args_list == [
            call(sectPrs[0], document_part_),
            call(sectPrs[1], document_part_),
            call(sectPrs[2], document_part_),
        ]
        assert section_lst == [section_, section_, section_]

    def it_can_access_its_Section_instances_by_slice(
        self, Section_, section_, document_part_
    ):
        document_elm = element(
            "w:document/w:body/(w:p/w:pPr/w:sectPr,w:p/w:pPr/w:sectPr,w:sectPr)"
        )
        sectPrs = document_elm.xpath("//w:sectPr")
        Section_.return_value = section_
        sections = Sections(document_elm, document_part_)

        section_lst = sections[1:9]

        assert Section_.call_args_list == [
            call(sectPrs[1], document_part_),
            call(sectPrs[2], document_part_),
        ]
        assert section_lst == [section_, section_]

    # fixture components ---------------------------------------------

    @pytest.fixture
    def document_part_(self, request):
        return instance_mock(request, DocumentPart)

    @pytest.fixture
    def Section_(self, request):
        return class_mock(request, "docx.section.Section")

    @pytest.fixture
    def section_(self, request):
        return instance_mock(request, Section)


class DescribeSection(object):
    def it_knows_when_it_displays_a_distinct_first_page_header(
        self, diff_first_header_get_fixture
    ):
        sectPr, expected_value = diff_first_header_get_fixture
        section = Section(sectPr, None)

        different_first_page_header_footer = section.different_first_page_header_footer

        assert different_first_page_header_footer is expected_value

    def it_can_change_whether_the_document_has_distinct_odd_and_even_headers(
        self, diff_first_header_set_fixture
    ):
        sectPr, value, expected_xml = diff_first_header_set_fixture
        section = Section(sectPr, None)

        section.different_first_page_header_footer = value

        assert sectPr.xml == expected_xml

    def it_provides_access_to_its_even_page_footer(
        self, document_part_, _Footer_, footer_
    ):
        sectPr = element("w:sectPr")
        _Footer_.return_value = footer_
        section = Section(sectPr, document_part_)

        footer = section.even_page_footer

        _Footer_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE
        )
        assert footer is footer_

    def it_provides_access_to_its_even_page_header(
        self, document_part_, _Header_, header_
    ):
        sectPr = element("w:sectPr")
        _Header_.return_value = header_
        section = Section(sectPr, document_part_)

        header = section.even_page_header

        _Header_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE
        )
        assert header is header_

    def it_provides_access_to_its_first_page_footer(
        self, document_part_, _Footer_, footer_
    ):
        sectPr = element("w:sectPr")
        _Footer_.return_value = footer_
        section = Section(sectPr, document_part_)

        footer = section.first_page_footer

        _Footer_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.FIRST_PAGE
        )
        assert footer is footer_

    def it_provides_access_to_its_first_page_header(
        self, document_part_, _Header_, header_
    ):
        sectPr = element("w:sectPr")
        _Header_.return_value = header_
        section = Section(sectPr, document_part_)

        header = section.first_page_header

        _Header_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.FIRST_PAGE
        )
        assert header is header_

    def it_provides_access_to_its_default_footer(
        self, document_part_, _Footer_, footer_
    ):
        sectPr = element("w:sectPr")
        _Footer_.return_value = footer_
        section = Section(sectPr, document_part_)

        footer = section.footer

        _Footer_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY
        )
        assert footer is footer_

    def it_provides_access_to_its_default_header(
        self, document_part_, _Header_, header_
    ):
        sectPr = element("w:sectPr")
        _Header_.return_value = header_
        section = Section(sectPr, document_part_)

        header = section.header

        _Header_.assert_called_once_with(
            sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY
        )
        assert header is header_

    def it_knows_its_start_type(self, start_type_get_fixture):
        sectPr, expected_start_type = start_type_get_fixture
        section = Section(sectPr, None)

        start_type = section.start_type

        assert start_type is expected_start_type

    def it_can_change_its_start_type(self, start_type_set_fixture):
        sectPr, new_start_type, expected_xml = start_type_set_fixture
        section = Section(sectPr, None)

        section.start_type = new_start_type

        assert section._sectPr.xml == expected_xml

    def it_knows_its_page_width(self, page_width_get_fixture):
        sectPr, expected_page_width = page_width_get_fixture
        section = Section(sectPr, None)

        page_width = section.page_width

        assert page_width == expected_page_width

    def it_can_change_its_page_width(self, page_width_set_fixture):
        sectPr, new_page_width, expected_xml = page_width_set_fixture
        section = Section(sectPr, None)

        section.page_width = new_page_width

        assert section._sectPr.xml == expected_xml

    def it_knows_its_page_height(self, page_height_get_fixture):
        sectPr, expected_page_height = page_height_get_fixture
        section = Section(sectPr, None)

        page_height = section.page_height

        assert page_height == expected_page_height

    def it_can_change_its_page_height(self, page_height_set_fixture):
        sectPr, new_page_height, expected_xml = page_height_set_fixture
        section = Section(sectPr, None)

        section.page_height = new_page_height

        assert section._sectPr.xml == expected_xml

    def it_knows_its_page_orientation(self, orientation_get_fixture):
        sectPr, expected_orientation = orientation_get_fixture
        section = Section(sectPr, None)

        orientation = section.orientation

        assert orientation is expected_orientation

    def it_can_change_its_orientation(self, orientation_set_fixture):
        sectPr, new_orientation, expected_xml = orientation_set_fixture
        section = Section(sectPr, None)

        section.orientation = new_orientation

        assert section._sectPr.xml == expected_xml

    def it_knows_its_page_margins(self, margins_get_fixture):
        sectPr, margin_prop_name, expected_value = margins_get_fixture
        section = Section(sectPr, None)

        value = getattr(section, margin_prop_name)

        assert value == expected_value

    def it_can_change_its_page_margins(self, margins_set_fixture):
        sectPr, margin_prop_name, new_value, expected_xml = margins_set_fixture
        section = Section(sectPr, None)

        setattr(section, margin_prop_name, new_value)

        assert section._sectPr.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:sectPr", False),
            ("w:sectPr/w:titlePg", True),
            ("w:sectPr/w:titlePg{w:val=0}", False),
            ("w:sectPr/w:titlePg{w:val=1}", True),
            ("w:sectPr/w:titlePg{w:val=true}", True),
        ]
    )
    def diff_first_header_get_fixture(self, request):
        sectPr_cxml, expected_value = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_value

    @pytest.fixture(
        params=[
            ("w:sectPr", True, "w:sectPr/w:titlePg"),
            ("w:sectPr/w:titlePg", False, "w:sectPr"),
            ("w:sectPr/w:titlePg{w:val=1}", True, "w:sectPr/w:titlePg"),
            ("w:sectPr/w:titlePg{w:val=off}", False, "w:sectPr"),
        ]
    )
    def diff_first_header_set_fixture(self, request):
        sectPr_cxml, value, expected_cxml = request.param
        sectPr = element(sectPr_cxml)
        expected_xml = xml(expected_cxml)
        return sectPr, value, expected_xml

    @pytest.fixture(
        params=[
            ("w:sectPr/w:pgMar{w:left=120}", "left_margin", 76200),
            ("w:sectPr/w:pgMar{w:right=240}", "right_margin", 152400),
            ("w:sectPr/w:pgMar{w:top=-360}", "top_margin", -228600),
            ("w:sectPr/w:pgMar{w:bottom=480}", "bottom_margin", 304800),
            ("w:sectPr/w:pgMar{w:gutter=600}", "gutter", 381000),
            ("w:sectPr/w:pgMar{w:header=720}", "header_distance", 457200),
            ("w:sectPr/w:pgMar{w:footer=840}", "footer_distance", 533400),
            ("w:sectPr/w:pgMar", "left_margin", None),
            ("w:sectPr", "top_margin", None),
        ]
    )
    def margins_get_fixture(self, request):
        sectPr_cxml, margin_prop_name, expected_value = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, margin_prop_name, expected_value

    @pytest.fixture(
        params=[
            ("w:sectPr", "left_margin", Inches(1), "w:sectPr/w:pgMar{w:left=1440}"),
            ("w:sectPr", "right_margin", Inches(0.5), "w:sectPr/w:pgMar{w:right=720}"),
            ("w:sectPr", "top_margin", Inches(-0.25), "w:sectPr/w:pgMar{w:top=-360}"),
            (
                "w:sectPr",
                "bottom_margin",
                Inches(0.75),
                "w:sectPr/w:pgMar{w:bottom=1080}",
            ),
            ("w:sectPr", "gutter", Inches(0.25), "w:sectPr/w:pgMar{w:gutter=360}"),
            (
                "w:sectPr",
                "header_distance",
                Inches(1.25),
                "w:sectPr/w:pgMar{w:header=1800}",
            ),
            (
                "w:sectPr",
                "footer_distance",
                Inches(1.35),
                "w:sectPr/w:pgMar{w:footer=1944}",
            ),
            ("w:sectPr", "left_margin", None, "w:sectPr/w:pgMar"),
            (
                "w:sectPr/w:pgMar{w:top=-360}",
                "top_margin",
                Inches(0.6),
                "w:sectPr/w:pgMar{w:top=864}",
            ),
        ]
    )
    def margins_set_fixture(self, request):
        sectPr_cxml, property_name, new_value, expected_cxml = request.param
        sectPr = element(sectPr_cxml)
        expected_xml = xml(expected_cxml)
        return sectPr, property_name, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("w:sectPr/w:pgSz{w:orient=landscape}", WD_ORIENT.LANDSCAPE),
            ("w:sectPr/w:pgSz{w:orient=portrait}", WD_ORIENT.PORTRAIT),
            ("w:sectPr/w:pgSz", WD_ORIENT.PORTRAIT),
            ("w:sectPr", WD_ORIENT.PORTRAIT),
        ]
    )
    def orientation_get_fixture(self, request):
        sectPr_cxml, expected_orientation = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_orientation

    @pytest.fixture(
        params=[
            (WD_ORIENT.LANDSCAPE, "w:sectPr/w:pgSz{w:orient=landscape}"),
            (WD_ORIENT.PORTRAIT, "w:sectPr/w:pgSz"),
            (None, "w:sectPr/w:pgSz"),
        ]
    )
    def orientation_set_fixture(self, request):
        new_orientation, expected_cxml = request.param
        sectPr = element("w:sectPr")
        expected_xml = xml(expected_cxml)
        return sectPr, new_orientation, expected_xml

    @pytest.fixture(
        params=[
            ("w:sectPr/w:pgSz{w:h=2880}", Inches(2)),
            ("w:sectPr/w:pgSz", None),
            ("w:sectPr", None),
        ]
    )
    def page_height_get_fixture(self, request):
        sectPr_cxml, expected_page_height = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_page_height

    @pytest.fixture(
        params=[
            (None, "w:sectPr/w:pgSz"),
            (Inches(2), "w:sectPr/w:pgSz{w:h=2880}"),
        ]
    )
    def page_height_set_fixture(self, request):
        new_page_height, expected_cxml = request.param
        sectPr = element("w:sectPr")
        expected_xml = xml(expected_cxml)
        return sectPr, new_page_height, expected_xml

    @pytest.fixture(
        params=[
            ("w:sectPr/w:pgSz{w:w=1440}", Inches(1)),
            ("w:sectPr/w:pgSz", None),
            ("w:sectPr", None),
        ]
    )
    def page_width_get_fixture(self, request):
        sectPr_cxml, expected_page_width = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_page_width

    @pytest.fixture(
        params=[
            (None, "w:sectPr/w:pgSz"),
            (Inches(4), "w:sectPr/w:pgSz{w:w=5760}"),
        ]
    )
    def page_width_set_fixture(self, request):
        new_page_width, expected_cxml = request.param
        sectPr = element("w:sectPr")
        expected_xml = xml(expected_cxml)
        return sectPr, new_page_width, expected_xml

    @pytest.fixture(
        params=[
            ("w:sectPr", WD_SECTION.NEW_PAGE),
            ("w:sectPr/w:type", WD_SECTION.NEW_PAGE),
            ("w:sectPr/w:type{w:val=continuous}", WD_SECTION.CONTINUOUS),
            ("w:sectPr/w:type{w:val=nextPage}", WD_SECTION.NEW_PAGE),
            ("w:sectPr/w:type{w:val=oddPage}", WD_SECTION.ODD_PAGE),
            ("w:sectPr/w:type{w:val=evenPage}", WD_SECTION.EVEN_PAGE),
            ("w:sectPr/w:type{w:val=nextColumn}", WD_SECTION.NEW_COLUMN),
        ]
    )
    def start_type_get_fixture(self, request):
        sectPr_cxml, expected_start_type = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_start_type

    @pytest.fixture(
        params=[
            (
                "w:sectPr/w:type{w:val=oddPage}",
                WD_SECTION.EVEN_PAGE,
                "w:sectPr/w:type{w:val=evenPage}",
            ),
            ("w:sectPr/w:type{w:val=nextPage}", None, "w:sectPr"),
            ("w:sectPr", None, "w:sectPr"),
            ("w:sectPr/w:type{w:val=continuous}", WD_SECTION.NEW_PAGE, "w:sectPr"),
            ("w:sectPr/w:type", WD_SECTION.NEW_PAGE, "w:sectPr"),
            (
                "w:sectPr/w:type",
                WD_SECTION.NEW_COLUMN,
                "w:sectPr/w:type{w:val=nextColumn}",
            ),
        ]
    )
    def start_type_set_fixture(self, request):
        initial_cxml, new_start_type, expected_cxml = request.param
        sectPr = element(initial_cxml)
        expected_xml = xml(expected_cxml)
        return sectPr, new_start_type, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def document_part_(self, request):
        return instance_mock(request, DocumentPart)

    @pytest.fixture
    def _Footer_(self, request):
        return class_mock(request, "docx.section._Footer")

    @pytest.fixture
    def footer_(self, request):
        return instance_mock(request, _Footer)

    @pytest.fixture
    def _Header_(self, request):
        return class_mock(request, "docx.section._Header")

    @pytest.fixture
    def header_(self, request):
        return instance_mock(request, _Header)


class Describe_BaseHeaderFooter(object):
    def it_knows_when_its_linked_to_the_previous_header_or_footer(
        self, is_linked_get_fixture, _has_definition_prop_
    ):
        has_definition, expected_value = is_linked_get_fixture
        _has_definition_prop_.return_value = has_definition
        header = _BaseHeaderFooter(None, None, None)

        is_linked = header.is_linked_to_previous

        assert is_linked is expected_value

    def it_can_change_whether_it_is_linked_to_previous_header_or_footer(
        self,
        is_linked_set_fixture,
        _has_definition_prop_,
        _drop_definition_,
        _add_definition_,
    ):
        has_definition, new_value, drop_calls, add_calls = is_linked_set_fixture
        _has_definition_prop_.return_value = has_definition
        header = _BaseHeaderFooter(None, None, None)

        header.is_linked_to_previous = new_value

        assert _drop_definition_.call_args_list == [call(header)] * drop_calls
        assert _add_definition_.call_args_list == [call(header)] * add_calls

    def it_provides_access_to_the_header_or_footer_part_for_BlockItemContainer(
        self, _get_or_add_definition_, header_part_
    ):
        # ---this override fulfills part of the BlockItemContainer subclass interface---
        _get_or_add_definition_.return_value = header_part_
        header = _BaseHeaderFooter(None, None, None)

        header_part = header.part

        _get_or_add_definition_.assert_called_once_with(header)
        assert header_part is header_part_

    def it_provides_access_to_the_hdr_or_ftr_element_to_help(
        self, _get_or_add_definition_, header_part_
    ):
        hdr = element("w:hdr")
        _get_or_add_definition_.return_value = header_part_
        header_part_.element = hdr
        header = _BaseHeaderFooter(None, None, None)

        hdr_elm = header._element

        _get_or_add_definition_.assert_called_once_with(header)
        assert hdr_elm is hdr

    def it_gets_the_definition_when_it_has_one(
        self, _has_definition_prop_, _definition_prop_, header_part_
    ):
        _has_definition_prop_.return_value = True
        _definition_prop_.return_value = header_part_
        header = _BaseHeaderFooter(None, None, None)

        header_part = header._get_or_add_definition()

        assert header_part is header_part_

    def but_it_gets_the_prior_definition_when_it_is_linked(
        self,
        _has_definition_prop_,
        _prior_headerfooter_prop_,
        prior_headerfooter_,
        header_part_,
    ):
        _has_definition_prop_.return_value = False
        _prior_headerfooter_prop_.return_value = prior_headerfooter_
        prior_headerfooter_._get_or_add_definition.return_value = header_part_
        header = _BaseHeaderFooter(None, None, None)

        header_part = header._get_or_add_definition()

        prior_headerfooter_._get_or_add_definition.assert_called_once_with()
        assert header_part is header_part_

    def and_it_adds_a_definition_when_it_is_linked_and_the_first_section(
        self,
        _has_definition_prop_,
        _prior_headerfooter_prop_,
        _add_definition_,
        header_part_,
    ):
        _has_definition_prop_.return_value = False
        _prior_headerfooter_prop_.return_value = None
        _add_definition_.return_value = header_part_
        header = _BaseHeaderFooter(None, None, None)

        header_part = header._get_or_add_definition()

        _add_definition_.assert_called_once_with(header)
        assert header_part is header_part_

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[(False, True), (True, False)])
    def is_linked_get_fixture(self, request):
        has_definition, expected_value = request.param
        return has_definition, expected_value

    @pytest.fixture(
        params=[
            (False, True, 0, 0),
            (True, False, 0, 0),
            (True, True, 1, 0),
            (False, False, 0, 1),
        ]
    )
    def is_linked_set_fixture(self, request):
        has_definition, new_value, drop_calls, add_calls = request.param
        return has_definition, new_value, drop_calls, add_calls

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _add_definition_(self, request):
        return method_mock(request, _BaseHeaderFooter, "_add_definition")

    @pytest.fixture
    def _definition_prop_(self, request):
        return property_mock(request, _BaseHeaderFooter, "_definition")

    @pytest.fixture
    def _drop_definition_(self, request):
        return method_mock(request, _BaseHeaderFooter, "_drop_definition")

    @pytest.fixture
    def _get_or_add_definition_(self, request):
        return method_mock(request, _BaseHeaderFooter, "_get_or_add_definition")

    @pytest.fixture
    def _has_definition_prop_(self, request):
        return property_mock(request, _BaseHeaderFooter, "_has_definition")

    @pytest.fixture
    def header_part_(self, request):
        return instance_mock(request, HeaderPart)

    @pytest.fixture
    def prior_headerfooter_(self, request):
        return instance_mock(request, _BaseHeaderFooter)

    @pytest.fixture
    def _prior_headerfooter_prop_(self, request):
        return property_mock(request, _BaseHeaderFooter, "_prior_headerfooter")


class Describe_Footer(object):
    def it_can_add_a_footer_part_to_help(self, document_part_, footer_part_):
        sectPr = element("w:sectPr{r:a=b}")
        document_part_.add_footer_part.return_value = footer_part_, "rId3"
        footer = _Footer(sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY)

        footer_part = footer._add_definition()

        document_part_.add_footer_part.assert_called_once_with()
        assert sectPr.xml == xml(
            "w:sectPr{r:a=b}/w:footerReference{w:type=default,r:id=rId3}"
        )
        assert footer_part is footer_part_

    def it_provides_access_to_its_footer_part_to_help(
        self, document_part_, footer_part_
    ):
        sectPr = element("w:sectPr/w:footerReference{w:type=even,r:id=rId3}")
        document_part_.footer_part.return_value = footer_part_
        footer = _Footer(sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE)

        footer_part = footer._definition

        document_part_.footer_part.assert_called_once_with("rId3")
        assert footer_part is footer_part_

    def it_can_drop_the_related_footer_part_to_help(self, document_part_):
        sectPr = element("w:sectPr{r:a=b}/w:footerReference{w:type=first,r:id=rId42}")
        footer = _Footer(sectPr, document_part_, WD_HEADER_FOOTER.FIRST_PAGE)

        footer._drop_definition()

        assert sectPr.xml == xml("w:sectPr{r:a=b}")
        document_part_.drop_rel.assert_called_once_with("rId42")

    def it_knows_when_it_has_a_definition_to_help(self, has_definition_fixture):
        sectPr, expected_value = has_definition_fixture
        footer = _Footer(sectPr, None, WD_HEADER_FOOTER.PRIMARY)

        has_definition = footer._has_definition

        assert has_definition is expected_value

    def it_provides_access_to_the_prior_Footer_to_help(
        self, request, document_part_, footer_
    ):
        doc_elm = element("w:document/(w:sectPr,w:sectPr)")
        prior_sectPr, sectPr = doc_elm[0], doc_elm[1]
        footer = _Footer(sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE)
        # ---mock must occur after construction of "real" footer---
        _Footer_ = class_mock(request, "docx.section._Footer", return_value=footer_)

        prior_footer = footer._prior_headerfooter

        _Footer_.assert_called_once_with(
            prior_sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE
        )
        assert prior_footer is footer_

    def but_it_returns_None_when_its_the_first_footer(self):
        doc_elm = element("w:document/w:sectPr")
        sectPr = doc_elm[0]
        footer = _Footer(sectPr, None, None)

        prior_footer = footer._prior_headerfooter

        assert prior_footer is None

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:sectPr", False),
            ("w:sectPr/w:footerReference{w:type=default}", True),
        ]
    )
    def has_definition_fixture(self, request):
        sectPr_cxml, expected_value = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def document_part_(self, request):
        return instance_mock(request, DocumentPart)

    @pytest.fixture
    def footer_(self, request):
        return instance_mock(request, _Footer)

    @pytest.fixture
    def footer_part_(self, request):
        return instance_mock(request, FooterPart)


class Describe_Header(object):
    def it_can_add_a_header_part_to_help(self, document_part_, header_part_):
        sectPr = element("w:sectPr{r:a=b}")
        document_part_.add_header_part.return_value = header_part_, "rId3"
        header = _Header(sectPr, document_part_, WD_HEADER_FOOTER.FIRST_PAGE)

        header_part = header._add_definition()

        document_part_.add_header_part.assert_called_once_with()
        assert sectPr.xml == xml(
            "w:sectPr{r:a=b}/w:headerReference{w:type=first,r:id=rId3}"
        )
        assert header_part is header_part_

    def it_provides_access_to_its_header_part_to_help(
        self, document_part_, header_part_
    ):
        sectPr = element("w:sectPr/w:headerReference{w:type=default,r:id=rId8}")
        document_part_.header_part.return_value = header_part_
        header = _Header(sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY)

        header_part = header._definition

        document_part_.header_part.assert_called_once_with("rId8")
        assert header_part is header_part_

    def it_can_drop_the_related_header_part_to_help(self, document_part_):
        sectPr = element("w:sectPr{r:a=b}/w:headerReference{w:type=even,r:id=rId42}")
        header = _Header(sectPr, document_part_, WD_HEADER_FOOTER.EVEN_PAGE)

        header._drop_definition()

        assert sectPr.xml == xml("w:sectPr{r:a=b}")
        document_part_.drop_header_part.assert_called_once_with("rId42")

    def it_knows_when_it_has_a_header_part_to_help(self, has_definition_fixture):
        sectPr, expected_value = has_definition_fixture
        header = _Header(sectPr, None, WD_HEADER_FOOTER.FIRST_PAGE)

        has_definition = header._has_definition

        assert has_definition is expected_value

    def it_provides_access_to_the_prior_Header_to_help(
        self, request, document_part_, header_
    ):
        doc_elm = element("w:document/(w:sectPr,w:sectPr)")
        prior_sectPr, sectPr = doc_elm[0], doc_elm[1]
        header = _Header(sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY)
        # ---mock must occur after construction of "real" header---
        _Header_ = class_mock(request, "docx.section._Header", return_value=header_)

        prior_header = header._prior_headerfooter

        _Header_.assert_called_once_with(
            prior_sectPr, document_part_, WD_HEADER_FOOTER.PRIMARY
        )
        assert prior_header is header_

    def but_it_returns_None_when_its_the_first_header(self):
        doc_elm = element("w:document/w:sectPr")
        sectPr = doc_elm[0]
        header = _Header(sectPr, None, None)

        prior_header = header._prior_headerfooter

        assert prior_header is None

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[("w:sectPr", False), ("w:sectPr/w:headerReference{w:type=first}", True)]
    )
    def has_definition_fixture(self, request):
        sectPr_cxml, expected_value = request.param
        sectPr = element(sectPr_cxml)
        return sectPr, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def document_part_(self, request):
        return instance_mock(request, DocumentPart)

    @pytest.fixture
    def header_(self, request):
        return instance_mock(request, _Header)

    @pytest.fixture
    def header_part_(self, request):
        return instance_mock(request, HeaderPart)



# encoding: utf-8

"""
Test suite for the docx.styles.style module
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from docx.enum.style import WD_STYLE_TYPE
from docx.styles.style import (
    BaseStyle,
    _CharacterStyle,
    _ParagraphStyle,
    _NumberingStyle,
    StyleFactory,
    _TableStyle,
)
from docx.text.font import Font
from docx.text.parfmt import ParagraphFormat

from ..unitutil.cxml import element, xml
from ..unitutil.mock import call, class_mock, function_mock, instance_mock


class DescribeStyleFactory(object):
    def it_constructs_the_right_type_of_style(self, factory_fixture):
        style_elm, StyleCls_, style_ = factory_fixture
        style = StyleFactory(style_elm)
        StyleCls_.assert_called_once_with(style_elm)
        assert style is style_

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=["paragraph", "character", "table", "numbering"])
    def factory_fixture(
        self,
        request,
        paragraph_style_,
        _ParagraphStyle_,
        character_style_,
        _CharacterStyle_,
        table_style_,
        _TableStyle_,
        numbering_style_,
        _NumberingStyle_,
    ):
        type_attr_val = request.param
        StyleCls_, style_mock = {
            "paragraph": (_ParagraphStyle_, paragraph_style_),
            "character": (_CharacterStyle_, character_style_),
            "table": (_TableStyle_, table_style_),
            "numbering": (_NumberingStyle_, numbering_style_),
        }[request.param]
        style_cxml = "w:style{w:type=%s}" % type_attr_val
        style_elm = element(style_cxml)
        return style_elm, StyleCls_, style_mock

    # fixture components -----------------------------------

    @pytest.fixture
    def _ParagraphStyle_(self, request, paragraph_style_):
        return class_mock(
            request, "docx.styles.style._ParagraphStyle", return_value=paragraph_style_
        )

    @pytest.fixture
    def paragraph_style_(self, request):
        return instance_mock(request, _ParagraphStyle)

    @pytest.fixture
    def _CharacterStyle_(self, request, character_style_):
        return class_mock(
            request, "docx.styles.style._CharacterStyle", return_value=character_style_
        )

    @pytest.fixture
    def character_style_(self, request):
        return instance_mock(request, _CharacterStyle)

    @pytest.fixture
    def _TableStyle_(self, request, table_style_):
        return class_mock(
            request, "docx.styles.style._TableStyle", return_value=table_style_
        )

    @pytest.fixture
    def table_style_(self, request):
        return instance_mock(request, _TableStyle)

    @pytest.fixture
    def _NumberingStyle_(self, request, numbering_style_):
        return class_mock(
            request, "docx.styles.style._NumberingStyle", return_value=numbering_style_
        )

    @pytest.fixture
    def numbering_style_(self, request):
        return instance_mock(request, _NumberingStyle)


class DescribeBaseStyle(object):
    def it_knows_its_style_id(self, id_get_fixture):
        style, expected_value = id_get_fixture
        assert style.style_id == expected_value

    def it_can_change_its_style_id(self, id_set_fixture):
        style, new_value, expected_xml = id_set_fixture
        style.style_id = new_value
        assert style._element.xml == expected_xml

    def it_knows_its_type(self, type_get_fixture):
        style, expected_value = type_get_fixture
        assert style.type == expected_value

    def it_knows_its_name(self, name_get_fixture):
        style, expected_value = name_get_fixture
        assert style.name == expected_value

    def it_can_change_its_name(self, name_set_fixture):
        style, new_value, expected_xml = name_set_fixture
        style.name = new_value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_a_builtin_style(self, builtin_get_fixture):
        style, expected_value = builtin_get_fixture
        assert style.builtin is expected_value

    def it_knows_whether_its_hidden(self, hidden_get_fixture):
        style, expected_value = hidden_get_fixture
        assert style.hidden == expected_value

    def it_can_change_whether_its_hidden(self, hidden_set_fixture):
        style, value, expected_xml = hidden_set_fixture
        style.hidden = value
        assert style._element.xml == expected_xml

    def it_knows_its_sort_order(self, priority_get_fixture):
        style, expected_value = priority_get_fixture
        assert style.priority == expected_value

    def it_can_change_its_sort_order(self, priority_set_fixture):
        style, value, expected_xml = priority_set_fixture
        style.priority = value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_unhide_when_used(self, unhide_get_fixture):
        style, expected_value = unhide_get_fixture
        assert style.unhide_when_used == expected_value

    def it_can_change_its_unhide_when_used_value(self, unhide_set_fixture):
        style, value, expected_xml = unhide_set_fixture
        style.unhide_when_used = value
        assert style._element.xml == expected_xml

    def it_knows_its_quick_style_setting(self, quick_get_fixture):
        style, expected_value = quick_get_fixture
        assert style.quick_style == expected_value

    def it_can_change_its_quick_style_setting(self, quick_set_fixture):
        style, new_value, expected_xml = quick_set_fixture
        style.quick_style = new_value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_locked(self, locked_get_fixture):
        style, expected_value = locked_get_fixture
        assert style.locked == expected_value

    def it_can_change_whether_its_locked(self, locked_set_fixture):
        style, value, expected_xml = locked_set_fixture
        style.locked = value
        assert style._element.xml == expected_xml

    def it_can_delete_itself_from_the_document(self, delete_fixture):
        style, styles, expected_xml = delete_fixture
        style.delete()
        assert styles.xml == expected_xml
        assert style._element is None

    # fixture --------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:style", True),
            ("w:style{w:customStyle=0}", True),
            ("w:style{w:customStyle=1}", False),
        ]
    )
    def builtin_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture
    def delete_fixture(self):
        styles = element("w:styles/w:style")
        style = BaseStyle(styles[0])
        expected_xml = xml("w:styles")
        return style, styles, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", False),
            ("w:style/w:semiHidden", True),
            ("w:style/w:semiHidden{w:val=0}", False),
            ("w:style/w:semiHidden{w:val=1}", True),
        ]
    )
    def hidden_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", True, "w:style/w:semiHidden"),
            ("w:style/w:semiHidden{w:val=0}", True, "w:style/w:semiHidden"),
            ("w:style/w:semiHidden{w:val=1}", True, "w:style/w:semiHidden"),
            ("w:style", False, "w:style"),
            ("w:style/w:semiHidden", False, "w:style"),
            ("w:style/w:semiHidden{w:val=1}", False, "w:style"),
        ]
    )
    def hidden_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", None),
            ("w:style{w:styleId=Foobar}", "Foobar"),
        ]
    )
    def id_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", "Foo", "w:style{w:styleId=Foo}"),
            ("w:style{w:styleId=Foo}", "Bar", "w:style{w:styleId=Bar}"),
            ("w:style{w:styleId=Bar}", None, "w:style"),
            ("w:style", None, "w:style"),
        ]
    )
    def id_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", False),
            ("w:style/w:locked", True),
            ("w:style/w:locked{w:val=0}", False),
            ("w:style/w:locked{w:val=1}", True),
        ]
    )
    def locked_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", True, "w:style/w:locked"),
            ("w:style/w:locked{w:val=0}", True, "w:style/w:locked"),
            ("w:style/w:locked{w:val=1}", True, "w:style/w:locked"),
            ("w:style", False, "w:style"),
            ("w:style/w:locked", False, "w:style"),
            ("w:style/w:locked{w:val=1}", False, "w:style"),
        ]
    )
    def locked_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style{w:type=table}", None),
            ("w:style{w:type=table}/w:name{w:val=Boofar}", "Boofar"),
            ("w:style{w:type=table}/w:name{w:val=heading 1}", "Heading 1"),
        ]
    )
    def name_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", "Foo", "w:style/w:name{w:val=Foo}"),
            ("w:style/w:name{w:val=Foo}", "Bar", "w:style/w:name{w:val=Bar}"),
            ("w:style/w:name{w:val=Bar}", None, "w:style"),
        ]
    )
    def name_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", None),
            ("w:style/w:uiPriority{w:val=42}", 42),
        ]
    )
    def priority_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", 42, "w:style/w:uiPriority{w:val=42}"),
            ("w:style/w:uiPriority{w:val=42}", 24, "w:style/w:uiPriority{w:val=24}"),
            ("w:style/w:uiPriority{w:val=24}", None, "w:style"),
        ]
    )
    def priority_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", False),
            ("w:style/w:qFormat", True),
            ("w:style/w:qFormat{w:val=0}", False),
            ("w:style/w:qFormat{w:val=on}", True),
        ]
    )
    def quick_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", True, "w:style/w:qFormat"),
            ("w:style/w:qFormat", False, "w:style"),
            ("w:style/w:qFormat", True, "w:style/w:qFormat"),
            ("w:style/w:qFormat{w:val=0}", False, "w:style"),
            ("w:style/w:qFormat{w:val=on}", True, "w:style/w:qFormat"),
        ]
    )
    def quick_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(
        params=[
            ("w:style", WD_STYLE_TYPE.PARAGRAPH),
            ("w:style{w:type=paragraph}", WD_STYLE_TYPE.PARAGRAPH),
            ("w:style{w:type=character}", WD_STYLE_TYPE.CHARACTER),
            ("w:style{w:type=numbering}", WD_STYLE_TYPE.LIST),
        ]
    )
    def type_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", False),
            ("w:style/w:unhideWhenUsed", True),
            ("w:style/w:unhideWhenUsed{w:val=0}", False),
            ("w:style/w:unhideWhenUsed{w:val=1}", True),
        ]
    )
    def unhide_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(
        params=[
            ("w:style", True, "w:style/w:unhideWhenUsed"),
            ("w:style/w:unhideWhenUsed", False, "w:style"),
            ("w:style/w:unhideWhenUsed{w:val=0}", True, "w:style/w:unhideWhenUsed"),
            ("w:style/w:unhideWhenUsed{w:val=1}", True, "w:style/w:unhideWhenUsed"),
            ("w:style/w:unhideWhenUsed{w:val=1}", False, "w:style"),
            ("w:style", False, "w:style"),
        ]
    )
    def unhide_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml


class Describe_CharacterStyle(object):
    def it_knows_which_style_it_is_based_on(self, base_get_fixture):
        style, StyleFactory_, StyleFactory_calls, base_style_ = base_get_fixture
        base_style = style.base_style

        assert StyleFactory_.call_args_list == StyleFactory_calls
        assert base_style == base_style_

    def it_can_change_its_base_style(self, base_set_fixture):
        style, value, expected_xml = base_set_fixture
        style.base_style = value
        assert style._element.xml == expected_xml

    def it_provides_access_to_its_font(self, font_fixture):
        style, Font_, font_ = font_fixture
        font = style.font
        Font_.assert_called_once_with(style._element)
        assert font is font_

    # fixture --------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:styles/(w:style{w:styleId=Foo},w:style/w:basedOn{w:val=Foo})", 1, 0),
            ("w:styles/(w:style{w:styleId=Foo},w:style/w:basedOn{w:val=Bar})", 1, -1),
            ("w:styles/w:style", 0, -1),
        ]
    )
    def base_get_fixture(self, request, StyleFactory_):
        styles_cxml, style_idx, base_style_idx = request.param
        styles = element(styles_cxml)
        style = _CharacterStyle(styles[style_idx])
        if base_style_idx >= 0:
            base_style = styles[base_style_idx]
            StyleFactory_calls = [call(base_style)]
            expected_value = StyleFactory_.return_value
        else:
            StyleFactory_calls = []
            expected_value = None
        return style, StyleFactory_, StyleFactory_calls, expected_value

    @pytest.fixture(
        params=[
            ("w:style", "Foo", "w:style/w:basedOn{w:val=Foo}"),
            ("w:style/w:basedOn{w:val=Foo}", "Bar", "w:style/w:basedOn{w:val=Bar}"),
            ("w:style/w:basedOn{w:val=Bar}", None, "w:style"),
        ]
    )
    def base_set_fixture(self, request, style_):
        style_cxml, base_style_id, expected_style_cxml = request.param
        style = _CharacterStyle(element(style_cxml))
        style_.style_id = base_style_id
        base_style = style_ if base_style_id is not None else None
        expected_xml = xml(expected_style_cxml)
        return style, base_style, expected_xml

    @pytest.fixture
    def font_fixture(self, Font_, font_):
        style = _CharacterStyle(element("w:style"))
        return style, Font_, font_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Font_(self, request, font_):
        return class_mock(request, "docx.styles.style.Font", return_value=font_)

    @pytest.fixture
    def font_(self, request):
        return instance_mock(request, Font)

    @pytest.fixture
    def style_(self, request):
        return instance_mock(request, BaseStyle)

    @pytest.fixture
    def StyleFactory_(self, request):
        return function_mock(request, "docx.styles.style.StyleFactory")


class Describe_ParagraphStyle(object):
    def it_knows_its_next_paragraph_style(self, next_get_fixture):
        style, expected_value = next_get_fixture
        assert style.next_paragraph_style == expected_value

    def it_can_change_its_next_paragraph_style(self, next_set_fixture):
        style, next_style, expected_xml = next_set_fixture
        style.next_paragraph_style = next_style
        assert style.element.xml == expected_xml

    def it_provides_access_to_its_paragraph_format(self, parfmt_fixture):
        style, ParagraphFormat_, paragraph_format_ = parfmt_fixture
        paragraph_format = style.paragraph_format
        ParagraphFormat_.assert_called_once_with(style._element)
        assert paragraph_format is paragraph_format_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("H1", "Body"),
            ("H2", "H2"),
            ("Body", "Body"),
            ("Foo", "Foo"),
        ]
    )
    def next_get_fixture(self, request):
        style_name, next_style_name = request.param
        styles = element(
            "w:styles/("
            "w:style{w:type=paragraph,w:styleId=H1}/w:next{w:val=Body},"
            "w:style{w:type=paragraph,w:styleId=H2}/w:next{w:val=Char},"
            "w:style{w:type=paragraph,w:styleId=Body},"
            "w:style{w:type=paragraph,w:styleId=Foo}/w:next{w:val=Bar},"
            "w:style{w:type=character,w:styleId=Char})"
        )
        style_names = ["H1", "H2", "Body", "Foo", "Char"]
        style_elm = styles[style_names.index(style_name)]
        next_style_elm = styles[style_names.index(next_style_name)]
        style = _ParagraphStyle(style_elm)
        if style_name == "H1":
            next_style = _ParagraphStyle(next_style_elm)
        else:
            next_style = style
        return style, next_style

    @pytest.fixture(
        params=[
            ("H", "B", "w:style{w:type=paragraph,w:styleId=H}/w:next{w:val=B}"),
            ("H", None, "w:style{w:type=paragraph,w:styleId=H}"),
            ("H", "H", "w:style{w:type=paragraph,w:styleId=H}"),
        ]
    )
    def next_set_fixture(self, request):
        style_name, next_style_name, style_cxml = request.param
        styles = element(
            "w:styles/("
            "w:style{w:type=paragraph,w:styleId=H},"
            "w:style{w:type=paragraph,w:styleId=B})"
        )
        style_elms = {"H": styles[0], "B": styles[1]}
        style = _ParagraphStyle(style_elms[style_name])
        next_style = (
            None
            if next_style_name is None
            else _ParagraphStyle(style_elms[next_style_name])
        )
        expected_xml = xml(style_cxml)
        return style, next_style, expected_xml

    @pytest.fixture
    def parfmt_fixture(self, ParagraphFormat_, paragraph_format_):
        style = _ParagraphStyle(element("w:style"))
        return style, ParagraphFormat_, paragraph_format_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ParagraphFormat_(self, request, paragraph_format_):
        return class_mock(
            request, "docx.styles.style.ParagraphFormat", return_value=paragraph_format_
        )

    @pytest.fixture
    def paragraph_format_(self, request):
        return instance_mock(request, ParagraphFormat)



# encoding: utf-8

"""
Test data builders for text XML elements
"""

from ...unitdata import BaseBuilder
from .shared import CT_StringBuilder


class CT_RowBuilder(BaseBuilder):
    __tag__ = "w:tr"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:w",)


class CT_TblBuilder(BaseBuilder):
    __tag__ = "w:tbl"
    __nspfxs__ = ("w",)
    __attrs__ = ()


class CT_TblGridBuilder(BaseBuilder):
    __tag__ = "w:tblGrid"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:w",)


class CT_TblGridColBuilder(BaseBuilder):
    __tag__ = "w:gridCol"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:w",)


class CT_TblPrBuilder(BaseBuilder):
    __tag__ = "w:tblPr"
    __nspfxs__ = ("w",)
    __attrs__ = ()


class CT_TblWidthBuilder(BaseBuilder):
    __tag__ = "w:tblW"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:w", "w:type")


class CT_TcBuilder(BaseBuilder):
    __tag__ = "w:tc"
    __nspfxs__ = ("w",)
    __attrs__ = ("w:id",)


class CT_TcPrBuilder(BaseBuilder):
    __tag__ = "w:tcPr"
    __nspfxs__ = ("w",)
    __attrs__ = ()


def a_gridCol():
    return CT_TblGridColBuilder()


def a_tbl():
    return CT_TblBuilder()


def a_tblGrid():
    return CT_TblGridBuilder()


def a_tblPr():
    return CT_TblPrBuilder()


def a_tblStyle():
    return CT_StringBuilder("w:tblStyle")


def a_tblW():
    return CT_TblWidthBuilder()


def a_tc():
    return CT_TcBuilder()


def a_tcPr():
    return CT_TcPrBuilder()


def a_tr():
    return CT_RowBuilder()



# encoding: utf-8

"""Unit test suite for the docx.text.paragraph module"""

from __future__ import absolute_import, division, print_function, unicode_literals

from docx.enum.style import WD_STYLE_TYPE
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.oxml.text.paragraph import CT_P
from docx.oxml.text.run import CT_R
from docx.parts.document import DocumentPart
from docx.text.paragraph import Paragraph
from docx.text.parfmt import ParagraphFormat
from docx.text.run import Run

import pytest

from ..unitutil.cxml import element, xml
from ..unitutil.mock import call, class_mock, instance_mock, method_mock, property_mock


class DescribeParagraph(object):
    def it_knows_its_paragraph_style(self, style_get_fixture):
        paragraph, style_id_, style_ = style_get_fixture
        style = paragraph.style
        paragraph.part.get_style.assert_called_once_with(
            style_id_, WD_STYLE_TYPE.PARAGRAPH
        )
        assert style is style_

    def it_can_change_its_paragraph_style(self, style_set_fixture):
        paragraph, value, expected_xml = style_set_fixture

        paragraph.style = value

        paragraph.part.get_style_id.assert_called_once_with(
            value, WD_STYLE_TYPE.PARAGRAPH
        )
        assert paragraph._p.xml == expected_xml

    def it_knows_the_text_it_contains(self, text_get_fixture):
        paragraph, expected_text = text_get_fixture
        assert paragraph.text == expected_text

    def it_can_replace_the_text_it_contains(self, text_set_fixture):
        paragraph, text, expected_text = text_set_fixture
        paragraph.text = text
        assert paragraph.text == expected_text

    def it_knows_its_alignment_value(self, alignment_get_fixture):
        paragraph, expected_value = alignment_get_fixture
        assert paragraph.alignment == expected_value

    def it_can_change_its_alignment_value(self, alignment_set_fixture):
        paragraph, value, expected_xml = alignment_set_fixture
        paragraph.alignment = value
        assert paragraph._p.xml == expected_xml

    def it_provides_access_to_its_paragraph_format(self, parfmt_fixture):
        paragraph, ParagraphFormat_, paragraph_format_ = parfmt_fixture
        paragraph_format = paragraph.paragraph_format
        ParagraphFormat_.assert_called_once_with(paragraph._element)
        assert paragraph_format is paragraph_format_

    def it_provides_access_to_the_runs_it_contains(self, runs_fixture):
        paragraph, Run_, r_, r_2_, run_, run_2_ = runs_fixture
        runs = paragraph.runs
        assert Run_.mock_calls == [call(r_, paragraph), call(r_2_, paragraph)]
        assert runs == [run_, run_2_]

    def it_can_add_a_run_to_itself(self, add_run_fixture):
        paragraph, text, style, style_prop_, expected_xml = add_run_fixture
        run = paragraph.add_run(text, style)
        assert paragraph._p.xml == expected_xml
        assert isinstance(run, Run)
        assert run._r is paragraph._p.r_lst[0]
        if style:
            style_prop_.assert_called_once_with(style)

    def it_can_insert_a_paragraph_before_itself(self, insert_before_fixture):
        text, style, paragraph_, add_run_calls = insert_before_fixture
        paragraph = Paragraph(None, None)

        new_paragraph = paragraph.insert_paragraph_before(text, style)

        paragraph._insert_paragraph_before.assert_called_once_with(paragraph)
        assert new_paragraph.add_run.call_args_list == add_run_calls
        assert new_paragraph.style == style
        assert new_paragraph is paragraph_

    def it_can_remove_its_content_while_preserving_formatting(self, clear_fixture):
        paragraph, expected_xml = clear_fixture
        _paragraph = paragraph.clear()
        assert paragraph._p.xml == expected_xml
        assert _paragraph is paragraph

    def it_inserts_a_paragraph_before_to_help(self, _insert_before_fixture):
        paragraph, body, expected_xml = _insert_before_fixture
        new_paragraph = paragraph._insert_paragraph_before()
        assert isinstance(new_paragraph, Paragraph)
        assert body.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:p", None, None, "w:p/w:r"),
            ("w:p", "foobar", None, 'w:p/w:r/w:t"foobar"'),
            ("w:p", None, "Strong", "w:p/w:r"),
            ("w:p", "foobar", "Strong", 'w:p/w:r/w:t"foobar"'),
        ]
    )
    def add_run_fixture(self, request, run_style_prop_):
        before_cxml, text, style, after_cxml = request.param
        paragraph = Paragraph(element(before_cxml), None)
        expected_xml = xml(after_cxml)
        return paragraph, text, style, run_style_prop_, expected_xml

    @pytest.fixture(
        params=[
            ("w:p/w:pPr/w:jc{w:val=center}", WD_ALIGN_PARAGRAPH.CENTER),
            ("w:p", None),
        ]
    )
    def alignment_get_fixture(self, request):
        cxml, expected_alignment_value = request.param
        paragraph = Paragraph(element(cxml), None)
        return paragraph, expected_alignment_value

    @pytest.fixture(
        params=[
            ("w:p", WD_ALIGN_PARAGRAPH.LEFT, "w:p/w:pPr/w:jc{w:val=left}"),
            (
                "w:p/w:pPr/w:jc{w:val=left}",
                WD_ALIGN_PARAGRAPH.CENTER,
                "w:p/w:pPr/w:jc{w:val=center}",
            ),
            ("w:p/w:pPr/w:jc{w:val=left}", None, "w:p/w:pPr"),
            ("w:p", None, "w:p/w:pPr"),
        ]
    )
    def alignment_set_fixture(self, request):
        initial_cxml, new_alignment_value, expected_cxml = request.param
        paragraph = Paragraph(element(initial_cxml), None)
        expected_xml = xml(expected_cxml)
        return paragraph, new_alignment_value, expected_xml

    @pytest.fixture(
        params=[
            ("w:p", "w:p"),
            ("w:p/w:pPr", "w:p/w:pPr"),
            ('w:p/w:r/w:t"foobar"', "w:p"),
            ('w:p/(w:pPr, w:r/w:t"foobar")', "w:p/w:pPr"),
        ]
    )
    def clear_fixture(self, request):
        initial_cxml, expected_cxml = request.param
        paragraph = Paragraph(element(initial_cxml), None)
        expected_xml = xml(expected_cxml)
        return paragraph, expected_xml

    @pytest.fixture(
        params=[
            (None, None),
            ("Foo", None),
            (None, "Bar"),
            ("Foo", "Bar"),
        ]
    )
    def insert_before_fixture(self, request, _insert_paragraph_before_, add_run_):
        text, style = request.param
        paragraph_ = _insert_paragraph_before_.return_value
        add_run_calls = [] if text is None else [call(text)]
        paragraph_.style = None
        return text, style, paragraph_, add_run_calls

    @pytest.fixture(params=[("w:body/w:p{id=42}", "w:body/(w:p,w:p{id=42})")])
    def _insert_before_fixture(self, request):
        body_cxml, expected_cxml = request.param
        body = element(body_cxml)
        paragraph = Paragraph(body[0], None)
        expected_xml = xml(expected_cxml)
        return paragraph, body, expected_xml

    @pytest.fixture
    def parfmt_fixture(self, ParagraphFormat_, paragraph_format_):
        paragraph = Paragraph(element("w:p"), None)
        return paragraph, ParagraphFormat_, paragraph_format_

    @pytest.fixture
    def runs_fixture(self, p_, Run_, r_, r_2_, runs_):
        paragraph = Paragraph(p_, None)
        run_, run_2_ = runs_
        return paragraph, Run_, r_, r_2_, run_, run_2_

    @pytest.fixture
    def style_get_fixture(self, part_prop_):
        style_id = "Foobar"
        p_cxml = "w:p/w:pPr/w:pStyle{w:val=%s}" % style_id
        paragraph = Paragraph(element(p_cxml), None)
        style_ = part_prop_.return_value.get_style.return_value
        return paragraph, style_id, style_

    @pytest.fixture(
        params=[
            ("w:p", "Heading 1", "Heading1", "w:p/w:pPr/w:pStyle{w:val=Heading1}"),
            (
                "w:p/w:pPr",
                "Heading 1",
                "Heading1",
                "w:p/w:pPr/w:pStyle{w:val=Heading1}",
            ),
            (
                "w:p/w:pPr/w:pStyle{w:val=Heading1}",
                "Heading 2",
                "Heading2",
                "w:p/w:pPr/w:pStyle{w:val=Heading2}",
            ),
            ("w:p/w:pPr/w:pStyle{w:val=Heading1}", "Normal", None, "w:p/w:pPr"),
            ("w:p", None, None, "w:p/w:pPr"),
        ]
    )
    def style_set_fixture(self, request, part_prop_):
        p_cxml, value, style_id, expected_cxml = request.param
        paragraph = Paragraph(element(p_cxml), None)
        part_prop_.return_value.get_style_id.return_value = style_id
        expected_xml = xml(expected_cxml)
        return paragraph, value, expected_xml

    @pytest.fixture(
        params=[
            ("w:p", ""),
            ("w:p/w:r", ""),
            ("w:p/w:r/w:t", ""),
            ('w:p/w:r/w:t"foo"', "foo"),
            ('w:p/w:r/(w:t"foo", w:t"bar")', "foobar"),
            ('w:p/w:r/(w:t"fo ", w:t"bar")', "fo bar"),
            ('w:p/w:r/(w:t"foo", w:tab, w:t"bar")', "foo\tbar"),
            ('w:p/w:r/(w:t"foo", w:br,  w:t"bar")', "foo\nbar"),
            ('w:p/w:r/(w:t"foo", w:cr,  w:t"bar")', "foo\nbar"),
        ]
    )
    def text_get_fixture(self, request):
        p_cxml, expected_text_value = request.param
        paragraph = Paragraph(element(p_cxml), None)
        return paragraph, expected_text_value

    @pytest.fixture
    def text_set_fixture(self):
        paragraph = Paragraph(element("w:p"), None)
        paragraph.add_run("must not appear in result")
        new_text_value = "foo\tbar\rbaz\n"
        expected_text_value = "foo\tbar\nbaz\n"
        return paragraph, new_text_value, expected_text_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def add_run_(self, request):
        return method_mock(request, Paragraph, "add_run")

    @pytest.fixture
    def document_part_(self, request):
        return instance_mock(request, DocumentPart)

    @pytest.fixture
    def _insert_paragraph_before_(self, request):
        return method_mock(request, Paragraph, "_insert_paragraph_before")

    @pytest.fixture
    def p_(self, request, r_, r_2_):
        return instance_mock(request, CT_P, r_lst=(r_, r_2_))

    @pytest.fixture
    def ParagraphFormat_(self, request, paragraph_format_):
        return class_mock(
            request,
            "docx.text.paragraph.ParagraphFormat",
            return_value=paragraph_format_,
        )

    @pytest.fixture
    def paragraph_format_(self, request):
        return instance_mock(request, ParagraphFormat)

    @pytest.fixture
    def part_prop_(self, request, document_part_):
        return property_mock(request, Paragraph, "part", return_value=document_part_)

    @pytest.fixture
    def Run_(self, request, runs_):
        run_, run_2_ = runs_
        return class_mock(
            request, "docx.text.paragraph.Run", side_effect=[run_, run_2_]
        )

    @pytest.fixture
    def r_(self, request):
        return instance_mock(request, CT_R)

    @pytest.fixture
    def r_2_(self, request):
        return instance_mock(request, CT_R)

    @pytest.fixture
    def run_style_prop_(self, request):
        return property_mock(request, Run, "style")

    @pytest.fixture
    def runs_(self, request):
        run_ = instance_mock(request, Run, name="run_")
        run_2_ = instance_mock(request, Run, name="run_2_")
        return run_, run_2_



# encoding: utf-8

"""
Custom element classes for shape-related elements like ``<w:inline>``
"""

from . import parse_xml
from .ns import nsdecls
from .simpletypes import (
    ST_Coordinate,
    ST_DrawingElementId,
    ST_PositiveCoordinate,
    ST_RelationshipId,
    XsdString,
    XsdToken,
)
from .xmlchemy import (
    BaseOxmlElement,
    OneAndOnlyOne,
    OptionalAttribute,
    RequiredAttribute,
    ZeroOrOne,
)


class CT_Blip(BaseOxmlElement):
    """
    ``<a:blip>`` element, specifies image source and adjustments such as
    alpha and tint.
    """

    embed = OptionalAttribute("r:embed", ST_RelationshipId)
    link = OptionalAttribute("r:link", ST_RelationshipId)


class CT_BlipFillProperties(BaseOxmlElement):
    """
    ``<pic:blipFill>`` element, specifies picture properties
    """

    blip = ZeroOrOne("a:blip", successors=("a:srcRect", "a:tile", "a:stretch"))


class CT_GraphicalObject(BaseOxmlElement):
    """
    ``<a:graphic>`` element, container for a DrawingML object
    """

    graphicData = OneAndOnlyOne("a:graphicData")


class CT_GraphicalObjectData(BaseOxmlElement):
    """
    ``<a:graphicData>`` element, container for the XML of a DrawingML object
    """

    pic = ZeroOrOne("pic:pic")
    uri = RequiredAttribute("uri", XsdToken)


class CT_Inline(BaseOxmlElement):
    """
    ``<w:inline>`` element, container for an inline shape.
    """

    extent = OneAndOnlyOne("wp:extent")
    docPr = OneAndOnlyOne("wp:docPr")
    graphic = OneAndOnlyOne("a:graphic")

    @classmethod
    def new(cls, cx, cy, shape_id, pic):
        """
        Return a new ``<wp:inline>`` element populated with the values passed
        as parameters.
        """
        inline = parse_xml(cls._inline_xml())
        inline.extent.cx = cx
        inline.extent.cy = cy
        inline.docPr.id = shape_id
        inline.docPr.name = "Picture %d" % shape_id
        inline.graphic.graphicData.uri = (
            "http://schemas.openxmlformats.org/drawingml/2006/picture"
        )
        inline.graphic.graphicData._insert_pic(pic)
        return inline

    @classmethod
    def new_pic_inline(cls, shape_id, rId, filename, cx, cy):
        """
        Return a new `wp:inline` element containing the `pic:pic` element
        specified by the argument values.
        """
        pic_id = 0  # Word doesn't seem to use this, but does not omit it
        pic = CT_Picture.new(pic_id, filename, rId, cx, cy)
        inline = cls.new(cx, cy, shape_id, pic)
        inline.graphic.graphicData._insert_pic(pic)
        return inline

    @classmethod
    def _inline_xml(cls):
        return (
            "<wp:inline %s>\n"
            '  <wp:extent cx="914400" cy="914400"/>\n'
            '  <wp:docPr id="666" name="unnamed"/>\n'
            "  <wp:cNvGraphicFramePr>\n"
            '    <a:graphicFrameLocks noChangeAspect="1"/>\n'
            "  </wp:cNvGraphicFramePr>\n"
            "  <a:graphic>\n"
            '    <a:graphicData uri="URI not set"/>\n'
            "  </a:graphic>\n"
            "</wp:inline>" % nsdecls("wp", "a", "pic", "r")
        )


class CT_NonVisualDrawingProps(BaseOxmlElement):
    """
    Used for ``<wp:docPr>`` element, and perhaps others. Specifies the id and
    name of a DrawingML drawing.
    """

    id = RequiredAttribute("id", ST_DrawingElementId)
    name = RequiredAttribute("name", XsdString)


class CT_NonVisualPictureProperties(BaseOxmlElement):
    """
    ``<pic:cNvPicPr>`` element, specifies picture locking and resize
    behaviors.
    """


class CT_Picture(BaseOxmlElement):
    """
    ``<pic:pic>`` element, a DrawingML picture
    """

    nvPicPr = OneAndOnlyOne("pic:nvPicPr")
    blipFill = OneAndOnlyOne("pic:blipFill")
    spPr = OneAndOnlyOne("pic:spPr")

    @classmethod
    def new(cls, pic_id, filename, rId, cx, cy):
        """
        Return a new ``<pic:pic>`` element populated with the minimal
        contents required to define a viable picture element, based on the
        values passed as parameters.
        """
        pic = parse_xml(cls._pic_xml())
        pic.nvPicPr.cNvPr.id = pic_id
        pic.nvPicPr.cNvPr.name = filename
        pic.blipFill.blip.embed = rId
        pic.spPr.cx = cx
        pic.spPr.cy = cy
        return pic

    @classmethod
    def _pic_xml(cls):
        return (
            "<pic:pic %s>\n"
            "  <pic:nvPicPr>\n"
            '    <pic:cNvPr id="666" name="unnamed"/>\n'
            "    <pic:cNvPicPr/>\n"
            "  </pic:nvPicPr>\n"
            "  <pic:blipFill>\n"
            "    <a:blip/>\n"
            "    <a:stretch>\n"
            "      <a:fillRect/>\n"
            "    </a:stretch>\n"
            "  </pic:blipFill>\n"
            "  <pic:spPr>\n"
            "    <a:xfrm>\n"
            '      <a:off x="0" y="0"/>\n'
            '      <a:ext cx="914400" cy="914400"/>\n'
            "    </a:xfrm>\n"
            '    <a:prstGeom prst="rect"/>\n'
            "  </pic:spPr>\n"
            "</pic:pic>" % nsdecls("pic", "a", "r")
        )


class CT_PictureNonVisual(BaseOxmlElement):
    """
    ``<pic:nvPicPr>`` element, non-visual picture properties
    """

    cNvPr = OneAndOnlyOne("pic:cNvPr")


class CT_Point2D(BaseOxmlElement):
    """
    Used for ``<a:off>`` element, and perhaps others. Specifies an x, y
    coordinate (point).
    """

    x = RequiredAttribute("x", ST_Coordinate)
    y = RequiredAttribute("y", ST_Coordinate)


class CT_PositiveSize2D(BaseOxmlElement):
    """
    Used for ``<wp:extent>`` element, and perhaps others later. Specifies the
    size of a DrawingML drawing.
    """

    cx = RequiredAttribute("cx", ST_PositiveCoordinate)
    cy = RequiredAttribute("cy", ST_PositiveCoordinate)


class CT_PresetGeometry2D(BaseOxmlElement):
    """
    ``<a:prstGeom>`` element, specifies an preset autoshape geometry, such
    as ``rect``.
    """


class CT_RelativeRect(BaseOxmlElement):
    """
    ``<a:fillRect>`` element, specifying picture should fill containing
    rectangle shape.
    """


class CT_ShapeProperties(BaseOxmlElement):
    """
    ``<pic:spPr>`` element, specifies size and shape of picture container.
    """

    xfrm = ZeroOrOne(
        "a:xfrm",
        successors=(
            "a:custGeom",
            "a:prstGeom",
            "a:ln",
            "a:effectLst",
            "a:effectDag",
            "a:scene3d",
            "a:sp3d",
            "a:extLst",
        ),
    )

    @property
    def cx(self):
        """
        Shape width as an instance of Emu, or None if not present.
        """
        xfrm = self.xfrm
        if xfrm is None:
            return None
        return xfrm.cx

    @cx.setter
    def cx(self, value):
        xfrm = self.get_or_add_xfrm()
        xfrm.cx = value

    @property
    def cy(self):
        """
        Shape height as an instance of Emu, or None if not present.
        """
        xfrm = self.xfrm
        if xfrm is None:
            return None
        return xfrm.cy

    @cy.setter
    def cy(self, value):
        xfrm = self.get_or_add_xfrm()
        xfrm.cy = value


class CT_StretchInfoProperties(BaseOxmlElement):
    """
    ``<a:stretch>`` element, specifies how picture should fill its containing
    shape.
    """


class CT_Transform2D(BaseOxmlElement):
    """
    ``<a:xfrm>`` element, specifies size and shape of picture container.
    """

    off = ZeroOrOne("a:off", successors=("a:ext",))
    ext = ZeroOrOne("a:ext", successors=())

    @property
    def cx(self):
        ext = self.ext
        if ext is None:
            return None
        return ext.cx

    @cx.setter
    def cx(self, value):
        ext = self.get_or_add_ext()
        ext.cx = value

    @property
    def cy(self):
        ext = self.ext
        if ext is None:
            return None
        return ext.cy

    @cy.setter
    def cy(self, value):
        ext = self.get_or_add_ext()
        ext.cy = value



# encoding: utf-8

"""
Exceptions used with python-docx.

The base exception class is PythonDocxError.
"""


class PythonDocxError(Exception):
    """
    Generic error class.
    """


class InvalidSpanError(PythonDocxError):
    """
    Raised when an invalid merge region is specified in a request to merge
    table cells.
    """


class InvalidXmlError(PythonDocxError):
    """
    Raised when invalid XML is encountered, such as on attempt to access a
    missing required child element
    """



# encoding: utf-8

"""
Step implementations for table cell-related features
"""

from __future__ import absolute_import, print_function, unicode_literals

from behave import given, then, when

from docx import Document

from helpers import test_docx


# given ===================================================


@given("a table cell")
def given_a_table_cell(context):
    table = Document(test_docx("tbl-2x2-table")).tables[0]
    context.cell = table.cell(0, 0)


# when =====================================================


@when("I add a 2 x 2 table into the first cell")
def when_I_add_a_2x2_table_into_the_first_cell(context):
    context.table_ = context.cell.add_table(2, 2)


@when("I assign a string to the cell text attribute")
def when_assign_string_to_cell_text_attribute(context):
    cell = context.cell
    text = "foobar"
    cell.text = text
    context.expected_text = text


# then =====================================================


@then("cell.tables[0] is a 2 x 2 table")
def then_cell_tables_0_is_a_2x2_table(context):
    cell = context.cell
    table = cell.tables[0]
    assert len(table.rows) == 2
    assert len(table.columns) == 2


@then("the cell contains the string I assigned")
def then_cell_contains_string_assigned(context):
    cell, expected_text = context.cell, context.expected_text
    text = cell.paragraphs[0].runs[0].text
    msg = "expected '%s', got '%s'" % (expected_text, text)
    assert text == expected_text, msg



# encoding: utf-8

"""
Test suite for docx.oxml.ns
"""

from __future__ import absolute_import, print_function, unicode_literals

import pytest

from docx.oxml.ns import NamespacePrefixedTag


class DescribeNamespacePrefixedTag(object):
    def it_behaves_like_a_string_when_you_want_it_to(self, nsptag):
        s = "- %s -" % nsptag
        assert s == "- a:foobar -"

    def it_knows_its_clark_name(self, nsptag, clark_name):
        assert nsptag.clark_name == clark_name

    def it_can_construct_from_a_clark_name(self, clark_name, nsptag):
        _nsptag = NamespacePrefixedTag.from_clark_name(clark_name)
        assert _nsptag == nsptag

    def it_knows_its_local_part(self, nsptag, local_part):
        assert nsptag.local_part == local_part

    def it_can_compose_a_single_entry_nsmap_for_itself(self, nsptag, namespace_uri_a):
        expected_nsmap = {"a": namespace_uri_a}
        assert nsptag.nsmap == expected_nsmap

    def it_knows_its_namespace_prefix(self, nsptag):
        assert nsptag.nspfx == "a"

    def it_knows_its_namespace_uri(self, nsptag, namespace_uri_a):
        assert nsptag.nsuri == namespace_uri_a

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def clark_name(self, namespace_uri_a, local_part):
        return "{%s}%s" % (namespace_uri_a, local_part)

    @pytest.fixture
    def local_part(self):
        return "foobar"

    @pytest.fixture
    def namespace_uri_a(self):
        return "http://schemas.openxmlformats.org/drawingml/2006/main"

    @pytest.fixture
    def nsptag(self, nsptag_str):
        return NamespacePrefixedTag(nsptag_str)

    @pytest.fixture
    def nsptag_str(self, local_part):
        return "a:%s" % local_part



#!/usr/bin/env python

import os
import re

from setuptools import find_packages, setup


def text_of(relpath):
    """
    Return string containing the contents of the file at *relpath* relative to
    this file.
    """
    thisdir = os.path.dirname(__file__)
    file_path = os.path.join(thisdir, os.path.normpath(relpath))
    with open(file_path) as f:
        text = f.read()
    return text


# Read the version from docx.__version__ without importing the package
# (and thus attempting to import packages it depends on that may not be
# installed yet)
version = re.search(r'__version__ = "([^"]+)"', text_of("docx/__init__.py")).group(1)


NAME = "python-docx-oss"
VERSION = version
DESCRIPTION = "Create and update Microsoft Word .docx files."
KEYWORDS = "docx office openxml word python-docx"
AUTHOR = "Ethan St. Lee"
AUTHOR_EMAIL = "ethan@ginolegaltech.cn"
URL = "https://github.com/lsaint/python-docx-oss"
LICENSE = text_of("LICENSE")
PACKAGES = find_packages(exclude=["tests", "tests.*"])
PACKAGE_DATA = {"docx": ["templates/*.xml", "templates/*.docx"]}

INSTALL_REQUIRES = ["lxml>=4.6.5"]
TEST_SUITE = "tests"
TESTS_REQUIRE = ["behave", "mock", "pyparsing", "pytest"]

CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Topic :: Office/Business :: Office Suites",
    "Topic :: Software Development :: Libraries",
]

LONG_DESCRIPTION = text_of("README.md") + "\n\n" + text_of("HISTORY.md")

ZIP_SAFE = False

params = {
    "name": NAME,
    "version": VERSION,
    "description": DESCRIPTION,
    "keywords": KEYWORDS,
    "long_description": LONG_DESCRIPTION,
    "long_description_content_type": "text/markdown",
    "author": AUTHOR,
    "author_email": AUTHOR_EMAIL,
    "url": URL,
    "license": LICENSE,
    "packages": PACKAGES,
    "package_data": PACKAGE_DATA,
    "install_requires": INSTALL_REQUIRES,
    "tests_require": TESTS_REQUIRE,
    "test_suite": TEST_SUITE,
    "classifiers": CLASSIFIERS,
    "zip_safe": ZIP_SAFE,
}

setup(**params)



# encoding: utf-8

"""
Unit test suite for the docx.opc.parts.customprops module
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import datetime, timedelta

import pytest

from docx.opc.customprops import CustomProperties
from docx.opc.parts.customprops import CustomPropertiesPart
from docx.oxml.customprops import CT_CustomProperties

from ...unitutil.mock import class_mock, instance_mock


class DescribeCustomPropertiesPart(object):
    def it_provides_access_to_its_custom_props_object(self, customprops_fixture):
        custom_properties_part, CustomProperties_ = customprops_fixture
        custom_properties = custom_properties_part.custom_properties
        CustomProperties_.assert_called_once_with(custom_properties_part.element)
        assert isinstance(custom_properties, CustomProperties)

    def it_can_create_a_default_custom_properties_part(self):
        custom_properties_part = CustomPropertiesPart.default(None)
        assert isinstance(custom_properties_part, CustomPropertiesPart)
        custom_properties = custom_properties_part.custom_properties
        assert len(custom_properties) == 0

    # fixtures ---------------------------------------------

    @pytest.fixture
    def customprops_fixture(self, element_, CustomProperties_):
        custom_properties_part = CustomPropertiesPart(None, None, element_, None)
        return custom_properties_part, CustomProperties_

    # fixture components -----------------------------------

    @pytest.fixture
    def CustomProperties_(self, request):
        return class_mock(request, "docx.opc.parts.customprops.CustomProperties")

    @pytest.fixture
    def element_(self, request):
        return instance_mock(request, CT_CustomProperties)



# encoding: utf-8

"""Test suite for the docx.oxml.text module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from docx.exceptions import InvalidSpanError
from docx.oxml import parse_xml
from docx.oxml.table import CT_Row, CT_Tc

from ..unitutil.cxml import element, xml
from ..unitutil.file import snippet_seq
from ..unitutil.mock import call, instance_mock, method_mock, property_mock


class DescribeCT_Row(object):
    def it_can_add_a_trPr(self, add_trPr_fixture):
        tr, expected_xml = add_trPr_fixture
        tr._add_trPr()
        assert tr.xml == expected_xml

    def it_raises_on_tc_at_grid_col(self, tc_raise_fixture):
        tr, idx = tc_raise_fixture
        with pytest.raises(ValueError):
            tr.tc_at_grid_col(idx)

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("w:tr", "w:tr/w:trPr"),
            ("w:tr/w:tblPrEx", "w:tr/(w:tblPrEx,w:trPr)"),
            ("w:tr/w:tc", "w:tr/(w:trPr,w:tc)"),
            ("w:tr/(w:sdt,w:del,w:tc)", "w:tr/(w:trPr,w:sdt,w:del,w:tc)"),
        ]
    )
    def add_trPr_fixture(self, request):
        tr_cxml, expected_cxml = request.param
        tr = element(tr_cxml)
        expected_xml = xml(expected_cxml)
        return tr, expected_xml

    @pytest.fixture(params=[(0, 0, 3), (1, 0, 1)])
    def tc_raise_fixture(self, request):
        snippet_idx, row_idx, col_idx = request.param
        tbl = parse_xml(snippet_seq("tbl-cells")[snippet_idx])
        tr = tbl.tr_lst[row_idx]
        return tr, col_idx


class DescribeCT_Tc(object):
    def it_can_merge_to_another_tc(
        self, tr_, _span_dimensions_, _tbl_, _grow_to_, top_tc_
    ):
        top_tr_ = tr_
        tc, other_tc = element("w:tc"), element("w:tc")
        top, left, height, width = 0, 1, 2, 3
        _span_dimensions_.return_value = top, left, height, width
        _tbl_.return_value.tr_lst = [tr_]
        tr_.tc_at_grid_col.return_value = top_tc_

        merged_tc = tc.merge(other_tc)

        _span_dimensions_.assert_called_once_with(tc, other_tc)
        top_tr_.tc_at_grid_col.assert_called_once_with(left)
        top_tc_._grow_to.assert_called_once_with(width, height)
        assert merged_tc is top_tc_

    def it_knows_its_extents_to_help(self, extents_fixture):
        tc, attr_name, expected_value = extents_fixture
        extent = getattr(tc, attr_name)
        assert extent == expected_value

    def it_calculates_the_dimensions_of_a_span_to_help(self, span_fixture):
        tc, other_tc, expected_dimensions = span_fixture
        dimensions = tc._span_dimensions(other_tc)
        assert dimensions == expected_dimensions

    def it_raises_on_invalid_span(self, span_raise_fixture):
        tc, other_tc = span_raise_fixture
        with pytest.raises(InvalidSpanError):
            tc._span_dimensions(other_tc)

    def it_can_grow_itself_to_help_merge(self, grow_to_fixture):
        tc, width, height, top_tc, expected_calls = grow_to_fixture
        tc._grow_to(width, height, top_tc)
        assert tc._span_to_width.call_args_list == expected_calls

    def it_can_extend_its_horz_span_to_help_merge(
        self, top_tc_, grid_span_, _move_content_to_, _swallow_next_tc_
    ):
        grid_span_.side_effect = [1, 3, 4]
        grid_width, vMerge = 4, "continue"
        tc = element("w:tc")

        tc._span_to_width(grid_width, top_tc_, vMerge)

        _move_content_to_.assert_called_once_with(tc, top_tc_)
        assert _swallow_next_tc_.call_args_list == [
            call(tc, grid_width, top_tc_),
            call(tc, grid_width, top_tc_),
        ]
        assert tc.vMerge == vMerge

    def it_can_swallow_the_next_tc_help_merge(self, swallow_fixture):
        tc, grid_width, top_tc, tr, expected_xml = swallow_fixture
        tc._swallow_next_tc(grid_width, top_tc)
        assert tr.xml == expected_xml

    def it_adds_cell_widths_on_swallow(self, add_width_fixture):
        tc, grid_width, top_tc, tr, expected_xml = add_width_fixture
        tc._swallow_next_tc(grid_width, top_tc)
        assert tr.xml == expected_xml

    def it_raises_on_invalid_swallow(self, swallow_raise_fixture):
        tc, grid_width, top_tc, tr = swallow_raise_fixture
        with pytest.raises(InvalidSpanError):
            tc._swallow_next_tc(grid_width, top_tc)

    def it_can_move_its_content_to_help_merge(self, move_fixture):
        tc, tc_2, expected_tc_xml, expected_tc_2_xml = move_fixture
        tc._move_content_to(tc_2)
        assert tc.xml == expected_tc_xml
        assert tc_2.xml == expected_tc_2_xml

    def it_raises_on_tr_above(self, tr_above_raise_fixture):
        tc = tr_above_raise_fixture
        with pytest.raises(ValueError):
            tc._tr_above

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            # both cells have a width
            (
                "w:tr/(w:tc/(w:tcPr/w:tcW{w:w=1440,w:type=dxa},w:p),"
                "w:tc/(w:tcPr/w:tcW{w:w=1440,w:type=dxa},w:p))",
                0,
                2,
                "w:tr/(w:tc/(w:tcPr/(w:tcW{w:w=2880,w:type=dxa},"
                "w:gridSpan{w:val=2}),w:p))",
            ),
            # neither have a width
            (
                "w:tr/(w:tc/w:p,w:tc/w:p)",
                0,
                2,
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))",
            ),
            # only second one has a width
            (
                "w:tr/(w:tc/w:p," "w:tc/(w:tcPr/w:tcW{w:w=1440,w:type=dxa},w:p))",
                0,
                2,
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))",
            ),
            # only first one has a width
            (
                "w:tr/(w:tc/(w:tcPr/w:tcW{w:w=1440,w:type=dxa},w:p)," "w:tc/w:p)",
                0,
                2,
                "w:tr/(w:tc/(w:tcPr/(w:tcW{w:w=1440,w:type=dxa},"
                "w:gridSpan{w:val=2}),w:p))",
            ),
        ]
    )
    def add_width_fixture(self, request):
        tr_cxml, tc_idx, grid_width, expected_tr_cxml = request.param
        tr = element(tr_cxml)
        tc = top_tc = tr[tc_idx]
        expected_tr_xml = xml(expected_tr_cxml)
        return tc, grid_width, top_tc, tr, expected_tr_xml

    @pytest.fixture(
        params=[
            (0, 0, 0, "top", 0),
            (2, 0, 1, "top", 0),
            (2, 1, 1, "top", 0),
            (4, 2, 1, "top", 1),
            (0, 0, 0, "left", 0),
            (1, 0, 1, "left", 2),
            (3, 1, 0, "left", 0),
            (3, 1, 1, "left", 2),
            (0, 0, 0, "bottom", 1),
            (1, 0, 0, "bottom", 1),
            (2, 0, 1, "bottom", 2),
            (4, 1, 1, "bottom", 3),
            (0, 0, 0, "right", 1),
            (1, 0, 0, "right", 2),
            (0, 0, 0, "right", 1),
            (4, 2, 1, "right", 3),
        ]
    )
    def extents_fixture(self, request):
        snippet_idx, row, col, attr_name, expected_value = request.param
        tbl = self._snippet_tbl(snippet_idx)
        tc = tbl.tr_lst[row].tc_lst[col]
        return tc, attr_name, expected_value

    @pytest.fixture(
        params=[
            (0, 0, 0, 2, 1),
            (0, 0, 1, 1, 2),
            (0, 1, 1, 2, 2),
            (1, 0, 0, 2, 2),
            (2, 0, 0, 2, 2),
            (2, 1, 2, 1, 2),
        ]
    )
    def grow_to_fixture(self, request, _span_to_width_):
        snippet_idx, row, col, width, height = request.param
        tbl = self._snippet_tbl(snippet_idx)
        tc = tbl.tr_lst[row].tc_lst[col]
        start = 0 if height == 1 else 1
        end = start + height
        expected_calls = [
            call(width, tc, None),
            call(width, tc, "restart"),
            call(width, tc, "continue"),
            call(width, tc, "continue"),
        ][start:end]
        return tc, width, height, None, expected_calls

    @pytest.fixture(
        params=[
            ("w:tc/w:p", "w:tc/w:p", "w:tc/w:p", "w:tc/w:p"),
            ("w:tc/w:p", "w:tc/w:p/w:r", "w:tc/w:p", "w:tc/w:p/w:r"),
            ("w:tc/w:p/w:r", "w:tc/w:p", "w:tc/w:p", "w:tc/w:p/w:r"),
            ("w:tc/(w:p/w:r,w:sdt)", "w:tc/w:p", "w:tc/w:p", "w:tc/(w:p/w:r,w:sdt)"),
            (
                "w:tc/(w:p/w:r,w:sdt)",
                "w:tc/(w:tbl,w:p)",
                "w:tc/w:p",
                "w:tc/(w:tbl,w:p/w:r,w:sdt)",
            ),
        ]
    )
    def move_fixture(self, request):
        tc_cxml, tc_2_cxml, expected_tc_cxml, expected_tc_2_cxml = request.param
        tc, tc_2 = element(tc_cxml), element(tc_2_cxml)
        expected_tc_xml = xml(expected_tc_cxml)
        expected_tc_2_xml = xml(expected_tc_2_cxml)
        return tc, tc_2, expected_tc_xml, expected_tc_2_xml

    @pytest.fixture(
        params=[
            (0, 0, 0, 0, 1, (0, 0, 1, 2)),
            (0, 0, 1, 2, 1, (0, 1, 3, 1)),
            (0, 2, 2, 1, 1, (1, 1, 2, 2)),
            (0, 1, 2, 1, 0, (1, 0, 1, 3)),
            (1, 0, 0, 1, 1, (0, 0, 2, 2)),
            (1, 0, 1, 0, 0, (0, 0, 1, 3)),
            (2, 0, 1, 2, 1, (0, 1, 3, 1)),
            (2, 0, 1, 1, 0, (0, 0, 2, 2)),
            (2, 1, 2, 0, 1, (0, 1, 2, 2)),
            (4, 0, 1, 0, 0, (0, 0, 1, 3)),
        ]
    )
    def span_fixture(self, request):
        snippet_idx, row, col, row_2, col_2, expected_value = request.param
        tbl = self._snippet_tbl(snippet_idx)
        tc = tbl.tr_lst[row].tc_lst[col]
        tc_2 = tbl.tr_lst[row_2].tc_lst[col_2]
        return tc, tc_2, expected_value

    @pytest.fixture(
        params=[
            (1, 0, 0, 1, 0),  # inverted-L horz
            (1, 1, 0, 0, 0),  # same in opposite order
            (2, 0, 2, 0, 1),  # inverted-L vert
            (5, 0, 1, 1, 0),  # tee-shape horz bar
            (5, 1, 0, 2, 1),  # same, opposite side
            (6, 1, 0, 0, 1),  # tee-shape vert bar
            (6, 0, 1, 1, 2),  # same, opposite side
        ]
    )
    def span_raise_fixture(self, request):
        snippet_idx, row, col, row_2, col_2 = request.param
        tbl = self._snippet_tbl(snippet_idx)
        tc = tbl.tr_lst[row].tc_lst[col]
        tc_2 = tbl.tr_lst[row_2].tc_lst[col_2]
        return tc, tc_2

    @pytest.fixture(
        params=[
            (
                "w:tr/(w:tc/w:p,w:tc/w:p)",
                0,
                2,
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))",
            ),
            (
                "w:tr/(w:tc/w:p,w:tc/w:p,w:tc/w:p)",
                1,
                2,
                "w:tr/(w:tc/w:p,w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))",
            ),
            (
                'w:tr/(w:tc/w:p/w:r/w:t"a",w:tc/w:p/w:r/w:t"b")',
                0,
                2,
                'w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p/w:r/w:t"a",'
                'w:p/w:r/w:t"b"))',
            ),
            (
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p),w:tc/w:p)",
                0,
                3,
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=3},w:p))",
            ),
            (
                "w:tr/(w:tc/w:p,w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))",
                0,
                3,
                "w:tr/(w:tc/(w:tcPr/w:gridSpan{w:val=3},w:p))",
            ),
        ]
    )
    def swallow_fixture(self, request):
        tr_cxml, tc_idx, grid_width, expected_tr_cxml = request.param
        tr = element(tr_cxml)
        tc = top_tc = tr[tc_idx]
        expected_tr_xml = xml(expected_tr_cxml)
        return tc, grid_width, top_tc, tr, expected_tr_xml

    @pytest.fixture(
        params=[
            ("w:tr/w:tc/w:p", 0, 2),
            ("w:tr/(w:tc/w:p,w:tc/(w:tcPr/w:gridSpan{w:val=2},w:p))", 0, 2),
        ]
    )
    def swallow_raise_fixture(self, request):
        tr_cxml, tc_idx, grid_width = request.param
        tr = element(tr_cxml)
        tc = top_tc = tr[tc_idx]
        return tc, grid_width, top_tc, tr

    @pytest.fixture(params=[(0, 0, 0), (4, 0, 0)])
    def tr_above_raise_fixture(self, request):
        snippet_idx, row_idx, col_idx = request.param
        tbl = parse_xml(snippet_seq("tbl-cells")[snippet_idx])
        tc = tbl.tr_lst[row_idx].tc_lst[col_idx]
        return tc

    # fixture components ---------------------------------------------

    @pytest.fixture
    def grid_span_(self, request):
        return property_mock(request, CT_Tc, "grid_span")

    @pytest.fixture
    def _grow_to_(self, request):
        return method_mock(request, CT_Tc, "_grow_to")

    @pytest.fixture
    def _move_content_to_(self, request):
        return method_mock(request, CT_Tc, "_move_content_to")

    @pytest.fixture
    def _span_dimensions_(self, request):
        return method_mock(request, CT_Tc, "_span_dimensions")

    @pytest.fixture
    def _span_to_width_(self, request):
        return method_mock(request, CT_Tc, "_span_to_width", autospec=False)

    def _snippet_tbl(self, idx):
        """
        Return a <w:tbl> element for snippet at *idx* in 'tbl-cells' snippet
        file.
        """
        return parse_xml(snippet_seq("tbl-cells")[idx])

    @pytest.fixture
    def _swallow_next_tc_(self, request):
        return method_mock(request, CT_Tc, "_swallow_next_tc")

    @pytest.fixture
    def _tbl_(self, request):
        return property_mock(request, CT_Tc, "_tbl")

    @pytest.fixture
    def top_tc_(self, request):
        return instance_mock(request, CT_Tc)

    @pytest.fixture
    def tr_(self, request):
        return instance_mock(request, CT_Row)



# encoding: utf-8

"""
The proxy class for an image part, and related objects.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import hashlib

from docx.image.image import Image
from docx.opc.part import Part
from docx.shared import Emu, Inches


class ImagePart(Part):
    """
    An image part. Corresponds to the target part of a relationship with type
    RELATIONSHIP_TYPE.IMAGE.
    """

    def __init__(self, partname, content_type, blob, image=None):
        super(ImagePart, self).__init__(partname, content_type, blob)
        self._image = image

    @property
    def default_cx(self):
        """
        Native width of this image, calculated from its width in pixels and
        horizontal dots per inch (dpi).
        """
        px_width = self.image.px_width
        horz_dpi = self.image.horz_dpi
        width_in_inches = px_width / horz_dpi
        return Inches(width_in_inches)

    @property
    def default_cy(self):
        """
        Native height of this image, calculated from its height in pixels and
        vertical dots per inch (dpi).
        """
        px_height = self.image.px_height
        horz_dpi = self.image.horz_dpi
        height_in_emu = 914400 * px_height / horz_dpi
        return Emu(height_in_emu)

    @property
    def filename(self):
        """
        Filename from which this image part was originally created. A generic
        name, e.g. 'image.png', is substituted if no name is available, for
        example when the image was loaded from an unnamed stream. In that
        case a default extension is applied based on the detected MIME type
        of the image.
        """
        if self._image is not None:
            return self._image.filename
        return "image.%s" % self.partname.ext

    @classmethod
    def from_image(cls, image, partname):
        """
        Return an |ImagePart| instance newly created from *image* and
        assigned *partname*.
        """
        return ImagePart(partname, image.content_type, image.blob, image)

    @property
    def image(self):
        if self._image is None:
            self._image = Image.from_blob(self.blob)
        return self._image

    @classmethod
    def load(cls, partname, content_type, blob, package):
        """
        Called by ``docx.opc.package.PartFactory`` to load an image part from
        a package being opened by ``Document(...)`` call.
        """
        return cls(partname, content_type, blob)

    @property
    def sha1(self):
        """
        SHA1 hash digest of the blob of this image part.
        """
        return hashlib.sha1(self._blob).hexdigest()



# encoding: utf-8

"""
Test suite for pptx.oxml.__init__.py module, primarily XML parser-related.
"""

from __future__ import print_function, unicode_literals

import pytest

from lxml import etree

from docx.oxml import OxmlElement, oxml_parser, parse_xml, register_element_cls
from docx.oxml.ns import qn
from docx.oxml.shared import BaseOxmlElement


class DescribeOxmlElement(object):
    def it_returns_an_lxml_element_with_matching_tag_name(self):
        element = OxmlElement("a:foo")
        assert isinstance(element, etree._Element)
        assert element.tag == (
            "{http://schemas.openxmlformats.org/drawingml/2006/main}foo"
        )

    def it_adds_supplied_attributes(self):
        element = OxmlElement("a:foo", {"a": "b", "c": "d"})
        assert etree.tostring(element) == (
            '<a:foo xmlns:a="http://schemas.openxmlformats.org/drawingml/200'
            '6/main" a="b" c="d"/>'
        ).encode("utf-8")

    def it_adds_additional_namespace_declarations_when_supplied(self):
        ns1 = "http://schemas.openxmlformats.org/drawingml/2006/main"
        ns2 = "other"
        element = OxmlElement("a:foo", nsdecls={"a": ns1, "x": ns2})
        assert len(element.nsmap.items()) == 2
        assert element.nsmap["a"] == ns1
        assert element.nsmap["x"] == ns2


class DescribeOxmlParser(object):
    def it_strips_whitespace_between_elements(self, whitespace_fixture):
        pretty_xml_text, stripped_xml_text = whitespace_fixture
        element = etree.fromstring(pretty_xml_text, oxml_parser)
        xml_text = etree.tostring(element, encoding="unicode")
        assert xml_text == stripped_xml_text

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def whitespace_fixture(self):
        pretty_xml_text = "<foø>\n" "  <bår>text</bår>\n" "</foø>\n"
        stripped_xml_text = "<foø><bår>text</bår></foø>"
        return pretty_xml_text, stripped_xml_text


class DescribeParseXml(object):
    def it_accepts_bytes_and_assumes_utf8_encoding(self, xml_bytes):
        parse_xml(xml_bytes)

    def it_accepts_unicode_providing_there_is_no_encoding_declaration(self):
        non_enc_decl = '<?xml version="1.0" standalone="yes"?>'
        enc_decl = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
        xml_body = "<foo><bar>føøbår</bar></foo>"
        # unicode body by itself doesn't raise
        parse_xml(xml_body)
        # adding XML decl without encoding attr doesn't raise either
        xml_text = "%s\n%s" % (non_enc_decl, xml_body)
        parse_xml(xml_text)
        # but adding encoding in the declaration raises ValueError
        xml_text = "%s\n%s" % (enc_decl, xml_body)
        with pytest.raises(ValueError):
            parse_xml(xml_text)

    def it_uses_registered_element_classes(self, xml_bytes):
        register_element_cls("a:foo", CustElmCls)
        element = parse_xml(xml_bytes)
        assert isinstance(element, CustElmCls)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def xml_bytes(self):
        return (
            '<a:foo xmlns:a="http://schemas.openxmlformats.org/drawingml/200'
            '6/main">\n'
            "  <a:bar>foøbår</a:bar>\n"
            "</a:foo>\n"
        ).encode("utf-8")


class DescribeRegisterElementCls(object):
    def it_determines_class_used_for_elements_with_matching_tagname(self, xml_text):
        register_element_cls("a:foo", CustElmCls)
        foo = parse_xml(xml_text)
        assert type(foo) is CustElmCls
        assert type(foo.find(qn("a:bar"))) is etree._Element

    # fixture components ---------------------------------------------

    @pytest.fixture
    def xml_text(self):
        return (
            '<a:foo xmlns:a="http://schemas.openxmlformats.org/drawingml/200'
            '6/main">\n'
            "  <a:bar>foøbår</a:bar>\n"
            "</a:foo>\n"
        )


# ===========================================================================
# static fixture
# ===========================================================================


class CustElmCls(BaseOxmlElement):
    pass



# encoding: utf-8

"""
Test data builders for DrawingML XML elements
"""

from ...unitdata import BaseBuilder


class CT_BlipBuilder(BaseBuilder):
    __tag__ = "a:blip"
    __nspfxs__ = ("a",)
    __attrs__ = ("r:embed", "r:link", "cstate")


class CT_BlipFillPropertiesBuilder(BaseBuilder):
    __tag__ = "pic:blipFill"
    __nspfxs__ = ("pic",)
    __attrs__ = ()


class CT_DrawingBuilder(BaseBuilder):
    __tag__ = "w:drawing"
    __nspfxs__ = ("w",)
    __attrs__ = ()


class CT_GraphicalObjectBuilder(BaseBuilder):
    __tag__ = "a:graphic"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_GraphicalObjectDataBuilder(BaseBuilder):
    __tag__ = "a:graphicData"
    __nspfxs__ = ("a",)
    __attrs__ = ("uri",)


class CT_GraphicalObjectFrameLockingBuilder(BaseBuilder):
    __tag__ = "a:graphicFrameLocks"
    __nspfxs__ = ("a",)
    __attrs__ = ("noChangeAspect",)


class CT_InlineBuilder(BaseBuilder):
    __tag__ = "wp:inline"
    __nspfxs__ = ("wp",)
    __attrs__ = ("distT", "distB", "distL", "distR")


class CT_NonVisualDrawingPropsBuilder(BaseBuilder):
    __nspfxs__ = ("wp",)
    __attrs__ = ("id", "name", "descr", "hidden", "title")

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_NonVisualDrawingPropsBuilder, self).__init__()


class CT_NonVisualGraphicFramePropertiesBuilder(BaseBuilder):
    __tag__ = "wp:cNvGraphicFramePr"
    __nspfxs__ = ("wp",)
    __attrs__ = ()


class CT_NonVisualPicturePropertiesBuilder(BaseBuilder):
    __tag__ = "pic:cNvPicPr"
    __nspfxs__ = ("pic",)
    __attrs__ = "preferRelativeResize"


class CT_PictureBuilder(BaseBuilder):
    __tag__ = "pic:pic"
    __nspfxs__ = ("pic",)
    __attrs__ = ()


class CT_PictureNonVisualBuilder(BaseBuilder):
    __tag__ = "pic:nvPicPr"
    __nspfxs__ = ("pic",)
    __attrs__ = ()


class CT_Point2DBuilder(BaseBuilder):
    __tag__ = "a:off"
    __nspfxs__ = ("a",)
    __attrs__ = ("x", "y")


class CT_PositiveSize2DBuilder(BaseBuilder):
    __nspfxs__ = ()
    __attrs__ = ("cx", "cy")

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_PositiveSize2DBuilder, self).__init__()


class CT_PresetGeometry2DBuilder(BaseBuilder):
    __tag__ = "a:prstGeom"
    __nspfxs__ = ("a",)
    __attrs__ = ("prst",)


class CT_RelativeRectBuilder(BaseBuilder):
    __tag__ = "a:fillRect"
    __nspfxs__ = ("a",)
    __attrs__ = ("l", "t", "r", "b")


class CT_ShapePropertiesBuilder(BaseBuilder):
    __tag__ = "pic:spPr"
    __nspfxs__ = ("pic", "a")
    __attrs__ = ("bwMode",)


class CT_StretchInfoPropertiesBuilder(BaseBuilder):
    __tag__ = "a:stretch"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_Transform2DBuilder(BaseBuilder):
    __tag__ = "a:xfrm"
    __nspfxs__ = ("a",)
    __attrs__ = ("rot", "flipH", "flipV")


def a_blip():
    return CT_BlipBuilder()


def a_blipFill():
    return CT_BlipFillPropertiesBuilder()


def a_cNvGraphicFramePr():
    return CT_NonVisualGraphicFramePropertiesBuilder()


def a_cNvPicPr():
    return CT_NonVisualPicturePropertiesBuilder()


def a_cNvPr():
    return CT_NonVisualDrawingPropsBuilder("pic:cNvPr")


def a_docPr():
    return CT_NonVisualDrawingPropsBuilder("wp:docPr")


def a_drawing():
    return CT_DrawingBuilder()


def a_fillRect():
    return CT_RelativeRectBuilder()


def a_graphic():
    return CT_GraphicalObjectBuilder()


def a_graphicData():
    return CT_GraphicalObjectDataBuilder()


def a_graphicFrameLocks():
    return CT_GraphicalObjectFrameLockingBuilder()


def a_pic():
    return CT_PictureBuilder()


def a_prstGeom():
    return CT_PresetGeometry2DBuilder()


def a_stretch():
    return CT_StretchInfoPropertiesBuilder()


def an_ext():
    return CT_PositiveSize2DBuilder("a:ext")


def an_extent():
    return CT_PositiveSize2DBuilder("wp:extent")


def an_inline():
    return CT_InlineBuilder()


def an_nvPicPr():
    return CT_PictureNonVisualBuilder()


def an_off():
    return CT_Point2DBuilder()


def an_spPr():
    return CT_ShapePropertiesBuilder()


def an_xfrm():
    return CT_Transform2DBuilder()
